#!/usr/bin/env python
"""
udocker unit tests: LocalRepository
"""

from unittest import TestCase, main
from unittest.mock import patch, mock_open, call
from udocker.container.localrepo import LocalRepository
from udocker.config import Config

BUILTIN = "builtins"
BOPEN = BUILTIN + '.open'
UDOCKER_TOPDIR = "/home/u1/.udocker"


class LocalRepositoryTestCase(TestCase):
    """Management of local repository of container
    images and extracted containers
    """

    def setUp(self):
        Config().getconf()
        Config().conf['topdir'] = UDOCKER_TOPDIR
        Config().conf['bindir'] = ""
        Config().conf['libdir'] = ""
        Config().conf['reposdir'] = ""
        Config().conf['layersdir'] = ""
        Config().conf['containersdir'] = ""
        Config().conf['homedir'] = "/home/u1"

    def tearDown(self):
        pass

    @patch('udocker.container.localrepo.FileUtil')
    def test_01_init(self, mock_fu):
        """Test01 LocalRepository() constructor."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        self.assertTrue(lrepo.topdir)
        self.assertTrue(lrepo.reposdir)
        self.assertTrue(lrepo.layersdir)
        self.assertTrue(lrepo.containersdir)
        self.assertTrue(lrepo.bindir)
        self.assertTrue(lrepo.libdir)
        self.assertTrue(lrepo.homedir)
        self.assertEqual(lrepo.topdir, UDOCKER_TOPDIR)
        self.assertEqual(lrepo.cur_repodir, "")
        self.assertEqual(lrepo.cur_tagdir, "")
        self.assertEqual(lrepo.cur_containerdir, "")
        self.assertTrue(mock_fu.register_prefix.called_count, 3)

    @patch('udocker.container.localrepo.FileUtil')
    def test_02_setup(self, mock_fu):
        """Test02 LocalRepository().setup()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None, None, None, None]
        newdir = "/home/u2/.udocker"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.setup(newdir)
        self.assertEqual(lrepo.topdir, newdir)

    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.os.makedirs')
    @patch('udocker.container.localrepo.FileUtil')
    def test_03_create_repo(self, mock_fu, mock_mkdir, mock_exists):
        """Test03 LocalRepository().create_repo()."""
        Config.conf['keystore'] = "tmp"
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [False, False, False, False,
                                   False, False, False, False]
        mock_mkdir.side_effect = [None, None, None, None,
                                  None, None, None, None]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.create_repo()
        self.assertTrue(status)
        self.assertTrue(mock_exists.call_count, 8)
        self.assertTrue(mock_mkdir.call_count, 8)

        Config.conf['keystore'] = "tmp"
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = OSError("fail")
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.create_repo()
        self.assertFalse(status)

    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_04_is_repo(self, mock_fu, mock_exists):
        """Test04 LocalRepository().is_repo()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [False, True, False, False,
                                   True]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.is_repo()
        self.assertTrue(mock_exists.call_count, 5)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, True, True, True,
                                   True]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.is_repo()
        self.assertTrue(mock_exists.call_count, 5)
        self.assertTrue(status)

    @patch('udocker.container.localrepo.FileUtil')
    def test_05_is_container_id(self, mock_fu):
        """Test05 LocalRepository().is_container_id()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        contid = ""
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.is_container_id(contid)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        contid = "d2578feb-acfc-37e0-8561-47335f85e46a"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.is_container_id(contid)
        self.assertTrue(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        contid = "d"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.is_container_id(contid)
        self.assertFalse(status)

    @patch.object(LocalRepository, '_protect')
    @patch('udocker.container.localrepo.FileUtil')
    def test_06_protect_container(self, mock_fu, mock_prot):
        """Test06 LocalRepository().protect_container()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        contid = "d2578feb-acfc-37e0-8561-47335f85e46a"
        mock_prot.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.protect_container(contid)
        self.assertTrue(status)
        self.assertTrue(mock_prot.called)

    @patch.object(LocalRepository, '_unprotect')
    @patch('udocker.container.localrepo.FileUtil')
    def test_07_unprotect_container(self, mock_fu, mock_unprot):
        """Test07 LocalRepository().unprotect_container()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        contid = "d2578feb-acfc-37e0-8561-47335f85e46a"
        mock_unprot.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.unprotect_container(contid)
        self.assertTrue(status)
        self.assertTrue(mock_unprot.called)

    @patch.object(LocalRepository, '_isprotected')
    @patch('udocker.container.localrepo.FileUtil')
    def test_08_isprotected_container(self, mock_fu, mock_isprot):
        """Test08 LocalRepository().isprotected_container()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        contid = "d2578feb-acfc-37e0-8561-47335f85e46a"
        mock_isprot.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.isprotected_container(contid)
        self.assertTrue(status)
        self.assertTrue(mock_isprot.called)

    @patch('udocker.container.localrepo.FileUtil')
    def test_09__protect(self, mock_fu):
        """Test09 LocalRepository()._protect()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        cdir = "/home/u1/.udocker/contid"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        with patch(BOPEN, mock_open()):
            status = lrepo._protect(cdir)
            self.assertTrue(status)

    @patch('udocker.container.localrepo.FileUtil')
    def test_10__unprotect(self, mock_fu):
        """Test10 LocalRepository()._unprotect()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        cdir = "/home/u1/.udocker/contid"
        mock_fu.return_value.remove.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._unprotect(cdir)
        self.assertTrue(status)
        self.assertTrue(mock_fu.return_value.remove.called)

    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_11__isprotected(self, mock_fu, mock_exists):
        """Test11 LocalRepository()._isprotected()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        cdir = "/home/u1/.udocker/contid"
        mock_exists.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._isprotected(cdir)
        self.assertTrue(status)
        self.assertTrue(mock_exists.called)

    @patch.object(LocalRepository, 'cd_container')
    @patch('udocker.container.localrepo.os.access')
    @patch('udocker.container.localrepo.os.path.isdir')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_12_iswriteable_container(self, mock_fu, mock_exists,
                                      mock_isdir, mock_access,
                                      mock_cdcont):
        """Test12 LocalRepository().iswriteable_container()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        container_id = "d2578feb-acfc-37e0-8561-47335f85e46a"
        mock_exists.return_value = False
        mock_cdcont.return_value = "/home/u1/.udocker/containerid"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.iswriteable_container(container_id)
        self.assertEqual(status, 2)
        self.assertTrue(mock_exists.called)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.return_value = True
        mock_isdir.return_value = False
        mock_cdcont.return_value = "/home/u1/.udocker/containerid"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.iswriteable_container(container_id)
        self.assertEqual(status, 3)
        self.assertTrue(mock_isdir.called)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.return_value = True
        mock_isdir.return_value = True
        mock_access.return_value = True
        mock_cdcont.return_value = "/home/u1/.udocker/containerid"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.iswriteable_container(container_id)
        self.assertEqual(status, 1)
        self.assertTrue(mock_access.called)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.return_value = True
        mock_isdir.return_value = True
        mock_access.return_value = False
        mock_cdcont.return_value = "/home/u1/.udocker/containerid"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.iswriteable_container(container_id)
        self.assertEqual(status, 0)

    @patch.object(LocalRepository, 'cd_container')
    @patch('udocker.container.localrepo.Uprocess.get_output')
    @patch('udocker.container.localrepo.FileUtil')
    def test_13_get_size(self, mock_fu, mock_getout, mock_cdcont):
        """Test13 LocalRepository().get_size()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        container_id = "d2578feb-acfc-37e0-8561-47335f85e46a"
        mock_getout.return_value = "1234 dd"
        mock_cdcont.return_value = "/home/u1/.udocker/containerid"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_size(container_id)
        self.assertEqual(status, 1234)
        self.assertTrue(mock_getout.called)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        container_id = "d2578feb-acfc-37e0-8561-47335f85e46a"
        mock_getout.return_value = ""
        mock_cdcont.return_value = "/home/u1/.udocker/containerid"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_size(container_id)
        self.assertEqual(status, -1)

    @patch('udocker.container.localrepo.os.listdir')
    @patch('udocker.container.localrepo.os.path.isdir')
    @patch('udocker.container.localrepo.FileUtil')
    def test_14_get_containers_list(self, mock_fu, mock_isdir,
                                    mock_listdir):
        """Test14 LocalRepository().get_containers_list()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_isdir.return_value = False
        mock_listdir.return_value = list()
        cdir = "/home/u1/.udocker/containers"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_containers_list()
        self.assertEqual(status, list())
        self.assertEqual(lrepo.containersdir, cdir)
        self.assertFalse(mock_listdir.called)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_isdir.return_value = True
        mock_listdir.return_value = list()
        cdir = "/home/u1/.udocker/containers"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_containers_list()
        self.assertEqual(status, list())
        self.assertTrue(mock_listdir.called)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_isdir.side_effect = [True, False]
        cdir = "/home/u1/.udocker/containers"
        mock_listdir.return_value = [cdir]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_containers_list()
        self.assertEqual(status, list())
        self.assertTrue(mock_listdir.called)
        self.assertTrue(mock_isdir.call_count, 2)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_isdir.side_effect = [True, True]
        cdir = "/home/u1/.udocker/containers"
        mock_listdir.return_value = ["a"]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        with patch(BOPEN, mock_open()):
            status = lrepo.get_containers_list()
            self.assertEqual(status, [cdir + "/" + "a"])
            self.assertTrue(mock_listdir.called)
            self.assertTrue(mock_isdir.call_count, 2)

    @patch.object(LocalRepository, 'del_container_name')
    @patch.object(LocalRepository, 'cd_container')
    @patch.object(LocalRepository, 'get_container_name')
    @patch.object(LocalRepository, 'get_containers_list')
    @patch('udocker.container.localrepo.FileUtil')
    def test_15_del_container(self, mock_fu,
                              mock_getlist, mock_getname,
                              mock_cdcont, mock_delname):
        """Test15 LocalRepository().del_container()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        cont_id = "d2578feb-acfc-37e0-8561-47335f85e46a"
        mock_cdcont.return_value = ""
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.del_container(cont_id)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        cont_id = "d2578feb-acfc-37e0-8561-47335f85e46a"
        cdirs = "/home/u1/.udocker/containers"
        contdir = cdirs + "/" + cont_id
        mock_cdcont.return_value = contdir
        mock_getlist.return_value = [contdir]
        mock_delname.return_value = None
        mock_getname.return_value = ["mycont"]
        mock_fu.return_value.remove.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.del_container(cont_id)
        self.assertFalse(status)
        self.assertTrue(mock_getlist.called)
        self.assertTrue(mock_delname.called)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        cont_id = "d2578feb-acfc-37e0-8561-47335f85e46a"
        cdirs = "/home/u1/.udocker/containers"
        contdir = cdirs + "/" + cont_id
        mock_cdcont.return_value = contdir
        mock_getlist.return_value = [contdir]
        mock_delname.return_value = None
        mock_getname.return_value = ["mycont"]
        mock_fu.return_value.remove.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.del_container(cont_id)
        self.assertTrue(status)
        self.assertTrue(mock_fu.return_value.remove.called)

    @patch.object(LocalRepository, 'get_containers_list')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_16_cd_container(self, mock_fu, mock_exists,
                             mock_getlist):
        """Test16 LocalRepository().cd_container()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        cont_id = "d2578feb-acfc-37e0-8561-47335f85e46a"
        cdirs = "/home/u1/.udocker/containers"
        contdir = cdirs + "/" + cont_id
        mock_getlist.return_value = [contdir]
        mock_exists.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.cd_container(cont_id)
        self.assertEqual(status, "")

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        cont_id = "d2578feb-acfc-37e0-8561-47335f85e46a"
        cdirs = "/home/u1/.udocker/containers"
        contdir = cdirs + "/" + cont_id
        mock_getlist.return_value = [contdir]
        mock_exists.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.cd_container(cont_id)
        self.assertEqual(status, contdir)

    @patch('udocker.container.localrepo.os.path.relpath')
    @patch('udocker.container.localrepo.os.symlink')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_17__symlink(self, mock_fu, mock_exists,
                         mock_symlink, mock_relpath):
        """Test17 LocalRepository()._symlink()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._symlink("EXISTINGFILE", "LINKFILE")
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.return_value = False
        mock_symlink.return_value = None
        mock_relpath.return_value = "cont/ROOT"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._symlink("EXISTINGFILE", "LINKFILE")
        self.assertTrue(status)

    @patch('udocker.container.localrepo.FileUtil')
    def test_18__name_is_valid(self, mock_fu):
        """Test18 LocalRepository()._name_is_valid().
        Check name alias validity.
        """
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        name = "lzskjghdlak"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._name_is_valid(name)
        self.assertTrue(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        name = "lzskjghd/lak"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._name_is_valid(name)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        name = ".lzsklak"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._name_is_valid(name)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        name = "]lzsklak"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._name_is_valid(name)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        name = "lzs[klak"
        status = lrepo._name_is_valid(name)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        name = "lzs klak"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._name_is_valid(name)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        name = "x" * 2049
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._name_is_valid(name)
        self.assertFalse(status)

    @patch('udocker.container.localrepo.os.path.exists')
    @patch.object(LocalRepository, '_symlink')
    @patch.object(LocalRepository, 'cd_container')
    @patch('udocker.container.localrepo.FileUtil')
    def test_18_set_container_name(self, mock_fu, mock_cd,
                                   mock_slink, mock_exists):
        """Test18 LocalRepository().set_container_name()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        container_id = "d2578feb-acfc-37e0-8561-47335f85e46a"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.set_container_name(container_id, "WRONG[/")
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_cd.return_value = "CONTAINERDIR"
        mock_exists.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.set_container_name(container_id, "RIGHT")
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_cd.return_value = "CONTAINERDIR"
        mock_exists.return_value = False
        mock_slink.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.set_container_name(container_id, "RIGHT")
        self.assertTrue(status)

    @patch.object(LocalRepository, '_name_is_valid')
    @patch('udocker.container.localrepo.os.path.islink')
    @patch('udocker.container.localrepo.FileUtil')
    def test_19_del_container_name(self, mock_fu,
                                   mock_islink, mock_nameisvalid):
        """Test19 LocalRepository().del_container_name()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        contname = "mycont"
        mock_nameisvalid.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.del_container_name(contname)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        contname = "mycont"
        mock_nameisvalid.return_value = True
        mock_islink.return_value = True
        mock_fu.return_value.remove.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.del_container_name(contname)
        self.assertTrue(status)

    @patch('udocker.container.localrepo.os.readlink')
    @patch('udocker.container.localrepo.os.path.isdir')
    @patch('udocker.container.localrepo.os.path.islink')
    @patch('udocker.container.localrepo.FileUtil')
    def test_20_get_container_id(self, mock_fu, mock_islink,
                                 mock_isdir, mock_readlink):
        """Test20 LocalRepository().get_container_id()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_container_id(None)
        self.assertEqual(status, "")

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_islink.return_value = True
        mock_readlink.return_value = "BASENAME"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_container_id("ALIASNAM")
        self.assertEqual(status, "BASENAME")

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_islink.return_value = False
        mock_isdir.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_container_id("ALIASNAM")
        self.assertEqual(status, "")

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_islink.return_value = False
        mock_isdir.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_container_id("ALIASNAM")
        self.assertEqual(status, "ALIASNAM")

    @patch('udocker.container.localrepo.os.readlink')
    @patch('udocker.container.localrepo.os.path.islink')
    @patch('udocker.container.localrepo.os.listdir')
    @patch('udocker.container.localrepo.os.path.isdir')
    @patch('udocker.container.localrepo.FileUtil')
    def test_21_get_container_name(self, mock_fu, mock_isdir,
                                   mock_listdir,
                                   mock_islink, mock_readlink):
        """Test21 LocalRepository().get_container_name()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_isdir.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        name_list = lrepo.get_container_name("IMAGE:TAG")
        self.assertEqual(name_list, list())

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_isdir.return_value = True
        mock_listdir.return_value = ['LINK']
        mock_islink.return_value = True
        mock_readlink.return_value = "/a/b/IMAGE:TAG"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        name_list = lrepo.get_container_name("IMAGE:TAG")
        self.assertEqual(name_list, ["LINK"])

    @patch('udocker.container.localrepo.os.makedirs')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_22_setup_container(self, mock_fu, mock_exists,
                                mock_makedirs):
        """Test22 LocalRepository().setup_container()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.setup_container("REPO", "TAG", "ID")
        self.assertEqual(status, "")

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.return_value = False
        mock_makedirs.return_value = None
        with patch(BOPEN, mock_open()):
            lrepo = LocalRepository(UDOCKER_TOPDIR)
            status = lrepo.setup_container("REPO", "TAG", "ID")
            self.assertEqual(status, lrepo.containersdir + "/ID")
            self.assertEqual(lrepo.cur_containerdir,
                             lrepo.containersdir + "/ID")

    @patch('udocker.container.localrepo.os.path.isfile')
    @patch('udocker.container.localrepo.FileUtil')
    def test_23__is_tag(self, mock_fu, mock_isfile):
        """Test23 LocalRepository()._is_tag()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_isfile.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._is_tag("tagdir")
        self.assertTrue(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_isfile.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._is_tag("tagdir")
        self.assertFalse(status)

    @patch('udocker.container.localrepo.FileUtil')
    def test_24_protect_imagerepo(self, mock_fu):
        """Test24 LocalRepository().protect_imagerepo()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        with patch(BOPEN, mock_open()) as mopen:
            lrepo = LocalRepository(UDOCKER_TOPDIR)
            lrepo.protect_imagerepo("IMAGE", "TAG")
            self.assertTrue(mopen.called)
            protect = lrepo.reposdir + "/IMAGE/TAG/PROTECT"
            self.assertEqual(mopen.call_args, call(protect, 'w', encoding='utf-8'))

    @patch('udocker.container.localrepo.FileUtil')
    @patch.object(LocalRepository, '_unprotect')
    def test_25_unprotect_imagerepo(self, mock_fu, mock_unprotect):
        """Test25 LocalRepository().unprotected_imagerepo()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.unprotect_imagerepo("IMAGE", "TAG")
        self.assertTrue(mock_unprotect.called)

    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_26_isprotected_imagerepo(self, mock_fu, mock_exists):
        """Test26 LocalRepository().isprotected_imagerepo()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.isprotected_imagerepo("IMAGE", "TAG")
        self.assertTrue(mock_exists.called)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        protect = lrepo.reposdir + "/IMAGE/TAG/PROTECT"
        self.assertEqual(mock_exists.call_args, call(protect))

    @patch.object(LocalRepository, '_is_tag')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_27_cd_imagerepo(self, mock_fu, mock_exists, mock_istag):
        """Test27 LocalRepository().cd_imagerepo()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        Config().conf['reposdir'] = "/tmp"
        mock_exists.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        out = lrepo.cd_imagerepo("", "")
        self.assertEqual(out, "")

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        Config().conf['reposdir'] = "/tmp"
        mock_exists.return_value = True
        mock_istag.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        out = lrepo.cd_imagerepo("IMAGE", "TAG")
        self.assertEqual(out, "/tmp/IMAGE/TAG")

    @patch('udocker.container.localrepo.os.path.islink')
    @patch('udocker.container.localrepo.os.path.isdir')
    @patch('udocker.container.localrepo.os.listdir')
    @patch('udocker.container.localrepo.FileUtil')
    def test_28__find(self, mock_fu, mock_listdir, mock_isdir, mock_islink):
        """Test28 LocalRepository()._find()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_fu.return_value.isdir.return_value = True
        mock_listdir.return_value = ["file"]
        mock_islink.return_value = True
        filename = "file"
        folder = "/tmp"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        out = lrepo._find(filename, folder)
        self.assertEqual(out, ["/tmp/file"])

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_islink.return_value = False
        mock_isdir.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        out = lrepo._find(filename, folder)
        self.assertEqual(out, [])

    @patch('udocker.container.localrepo.os.path.islink')
    @patch('udocker.container.localrepo.os.path.isdir')
    @patch('udocker.container.localrepo.os.listdir')
    @patch('udocker.container.localrepo.FileUtil')
    def test_29__inrepository(self, mock_fu, mock_listdir,
                              mock_isdir, mock_islink):
        """Test29 LocalRepository()._inrepository()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_fu.return_value.isdir.return_value = True
        mock_listdir.return_value = ["file"]
        mock_islink.return_value = True
        filename = "file"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.reposdir = "/tmp"
        out = lrepo._inrepository(filename)
        self.assertEqual(out, ["/tmp/file"])

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_islink.return_value = False
        mock_isdir.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        out = lrepo._inrepository(filename)
        self.assertEqual(out, [])

    @patch('udocker.container.localrepo.os.readlink')
    @patch('udocker.container.localrepo.os.path.islink')
    @patch('udocker.container.localrepo.os.listdir')
    @patch.object(LocalRepository, '_inrepository')
    @patch('udocker.container.localrepo.FileUtil')
    def test_30__remove_layers(self, mock_fu, mock_in, mock_listdir,
                               mock_islink, mock_readlink):
        """Test30 LocalRepository()._remove_layers()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_listdir.return_value = []
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._remove_layers("TAG_DIR", False)
        self.assertTrue(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_listdir.return_value = ["FILE1,", "FILE2"]
        mock_islink.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._remove_layers("TAG_DIR", False)
        self.assertTrue(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_islink.return_value = True
        mock_readlink.return_value = "REALFILE"
        mock_fu.return_value.remove.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._remove_layers("TAG_DIR", False)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_fu.return_value.remove.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._remove_layers("TAG_DIR", False)
        self.assertTrue(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_fu.return_value.remove.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._remove_layers("TAG_DIR", True)
        self.assertTrue(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_fu.return_value.remove.return_value = False
        mock_in.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._remove_layers("TAG_DIR", True)
        self.assertTrue(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_fu.return_value.remove.return_value = False
        mock_in.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._remove_layers("TAG_DIR", False)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_fu.return_value.remove.return_value = False
        mock_in.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._remove_layers("TAG_DIR", True)
        self.assertTrue(status)

    @patch.object(LocalRepository, 'cd_imagerepo')
    @patch.object(LocalRepository, '_remove_layers')
    @patch('udocker.container.localrepo.FileUtil')
    def test_31_del_imagerepo(self, mock_fu, mock_rmlayers, mock_cd):
        """Test31 LocalRepository()._del_imagerepo()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_cd.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.del_imagerepo("IMAGE", "TAG", False)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_cd.return_value = True
        mock_fu.return_value.remove.return_value = True
        mock_rmlayers.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = "XXXX"
        lrepo.cur_tagdir = "XXXX"
        status = lrepo.del_imagerepo("IMAGE", "TAG", False)
        self.assertEqual(lrepo.cur_repodir, "")
        self.assertEqual(lrepo.cur_tagdir, "")
        self.assertTrue(status)

    def _sideffect_test_32(self, arg):
        """Side effect for isdir on test 23 _get_tags()."""
        if self.itr < 3:
            self.itr += 1
            return False

        return True

    @patch('udocker.container.localrepo.os.path.isdir')
    @patch('udocker.container.localrepo.os.listdir')
    @patch.object(LocalRepository, '_is_tag')
    @patch('udocker.container.localrepo.FileUtil')
    def test_32__get_tags(self, mock_fu, mock_is,
                          mock_listdir, mock_isdir):
        """Test32 LocalRepository()._get_tags()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_isdir.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._get_tags("CONTAINERS_DIR")
        self.assertEqual(status, [])

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_isdir.return_value = True
        mock_listdir.return_value = []
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._get_tags("CONTAINERS_DIR")
        self.assertEqual(status, [])

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_isdir.return_value = True
        mock_listdir.return_value = ["FILE1", "FILE2"]
        mock_is.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._get_tags("CONTAINERS_DIR")
        expected_status = [('CONTAINERS_DIR', 'FILE1'),
                           ('CONTAINERS_DIR', 'FILE2')]
        self.assertEqual(status, expected_status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_isdir.return_value = True
        mock_listdir.return_value = ["FILE1", "FILE2"]
        mock_is.return_value = False
        self.itr = 0
        mock_isdir.side_effect = self._sideffect_test_32
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._get_tags("CONTAINERS_DIR")
        expected_status = [('CONTAINERS_DIR', 'FILE1'),
                           ('CONTAINERS_DIR', 'FILE2')]
        self.assertEqual(self.itr, 2)
        self.assertEqual(status, [])

    @patch.object(LocalRepository, '_get_tags')
    @patch('udocker.container.localrepo.FileUtil')
    def test_33_get_imagerepos(self, mock_fu, mock_gtags):
        """Test33 LocalRepository().get_imagerepos()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_gtags.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.get_imagerepos()
        self.assertTrue(mock_gtags.called)

    @patch.object(LocalRepository, 'cd_imagerepo')
    @patch('udocker.container.localrepo.os.path.islink')
    @patch('udocker.container.localrepo.os.listdir')
    @patch('udocker.container.localrepo.FileUtil')
    def test_34_get_layers(self, mock_fu, mock_ldir,
                           mock_islink, mock_cd):
        """Test34 LocalRepository().get_layers()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_ldir.return_value = ""
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_layers("", "")
        self.assertTrue(mock_cd.called)
        self.assertEqual(status, list())

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_ldir.return_value = ["f1"]
        mock_cd.return_value = "IMAGE/TAG"
        mock_islink.return_value = True
        mock_fu.return_value.size.return_value = 123
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_layers("IMAGE", "TAG")
        self.assertTrue(mock_ldir.called)
        self.assertTrue(mock_islink.called)
        self.assertEqual(status, [("IMAGE/TAG/f1", 123)])

    @patch.object(LocalRepository, '_symlink')
    @patch('udocker.container.localrepo.os.path.basename')
    @patch('udocker.container.localrepo.os.path.islink')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_35_add_image_layer(self, mock_fu, mock_exists,
                                mock_islink, mock_base, mock_symln):
        """Test35 LocalRepository().add_image_layer()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = ""
        lrepo.cur_tagdir = ""
        status = lrepo.add_image_layer("FILE")
        self.assertFalse(status)
        self.assertFalse(mock_exists.called)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [False, False]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = "IMAGE"
        lrepo.cur_tagdir = "TAG"
        status = lrepo.add_image_layer("FILE")
        self.assertFalse(status)
        self.assertTrue(mock_exists.call_count, 1)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, False]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = "IMAGE"
        lrepo.cur_tagdir = "TAG"
        status = lrepo.add_image_layer("FILE")
        self.assertFalse(status)
        self.assertTrue(mock_exists.call_count, 2)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, True]
        mock_base.return_value = "file1"
        mock_islink.return_value = True
        mock_fu.return_value.remove.return_value = None
        mock_symln.return_value = None
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = "IMAGE"
        lrepo.cur_tagdir = "TAG"
        status = lrepo.add_image_layer("FILE")
        self.assertTrue(status)
        self.assertTrue(mock_base.called)
        self.assertTrue(mock_fu.return_value.remove.called)
        self.assertTrue(mock_islink.called)
        self.assertTrue(mock_symln.called)

    @patch('udocker.container.localrepo.os.makedirs')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_36_setup_imagerepo(self, mock_fu, mock_exists,
                                mock_mkdirs):
        """Test36 LocalRepository().setup_imagerepo()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.setup_imagerepo("")
        self.assertEqual(status, None)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.return_value = True
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.setup_imagerepo("IMAGE")
        expected_directory = lrepo.reposdir + "/IMAGE"
        self.assertEqual(lrepo.cur_repodir, expected_directory)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.setup_imagerepo("IMAGE")
        expected_directory = lrepo.reposdir + "/IMAGE"
        self.assertTrue(mock_mkdirs.called)
        self.assertEqual(lrepo.cur_repodir, expected_directory)
        self.assertTrue(status)

    @patch('udocker.container.localrepo.os.makedirs')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_37_setup_tag(self, mock_fu, mock_exists, mock_mkdirs):
        """Test37 LocalRepository().setup_tag()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = lrepo.reposdir + "/IMAGE"
        with patch(BOPEN, mock_open()) as mopen:
            status = lrepo.setup_tag("NEWTAG")
            self.assertTrue(mock_mkdirs.called)
            expected_directory = lrepo.reposdir + "/IMAGE/NEWTAG"
            self.assertEqual(lrepo.cur_tagdir, expected_directory)
            self.assertTrue(mopen.called)
            self.assertTrue(status)

    @patch('udocker.container.localrepo.os.listdir')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_38_set_version(self, mock_fu, mock_exists,
                            mock_listdir):
        """Test38 LocalRepository().set_version()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.set_version("v1")
        self.assertFalse(mock_exists.called)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [False, False]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = lrepo.reposdir + "/IMAGE"
        lrepo.cur_tagdir = lrepo.cur_repodir + "/TAG"
        status = lrepo.set_version("v1")
        self.assertTrue(mock_exists.call_count, 1)
        self.assertFalse(mock_listdir.called)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, False]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = lrepo.reposdir + "/IMAGE"
        lrepo.cur_tagdir = lrepo.cur_repodir + "/TAG"
        status = lrepo.set_version("v1")
        self.assertTrue(mock_exists.call_count, 2)
        self.assertFalse(mock_listdir.called)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, True, True, True]
        mock_listdir.return_value = ["file1"]
        mock_fu.return_value.remove.side_effect = \
            [None, None]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.set_version("v2")
        self.assertFalse(status)
        self.assertTrue(mock_exists.call_count, 3)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, True, False, False]
        mock_listdir.return_value = ["file1"]
        mock_fu.return_value.remove.side_effect = \
            [None, None]
        with patch(BOPEN, mock_open()) as mopen:
            lrepo = LocalRepository(UDOCKER_TOPDIR)
            status = lrepo.set_version("v2")
            # self.assertTrue(status)
            self.assertTrue(mock_exists.call_count, 4)
            # self.assertTrue(mopen.called)

    @patch.object(LocalRepository, 'load_json')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_39__get_image_attr_v1(self, mock_fu, mock_exists,
                                   mock_loadjson):
        """Test39 LocalRepository()._get_image_attributes_v1()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_loadjson.return_value = None
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._get_image_attributes_v1("/cont")
        self.assertEqual(status, (None, None))

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_loadjson.return_value = ["lname1"]
        mock_exists.side_effect = [False, False]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._get_image_attributes_v1("/cont")
        self.assertEqual(status, (None, None))

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_loadjson.side_effect = [["lname1"], ["lname1"]]
        mock_exists.side_effect = [True, True]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._get_image_attributes_v1("/cont")
        self.assertEqual(status, (['lname1'], ['/cont/lname1.layer']))

    @patch('udocker.container.localrepo.json.loads')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_40__get_image_attr_v2_s1(self, mock_fu, mock_exists,
                                      mock_jload):
        """Test40 LocalRepository()._get_image_attributes_v2_s1()."""
        manifest = {
            "fsLayers": ({"blobSum": "foolayername"},),
            "history": ({"v1Compatibility": '["foojsonstring"]'},)
        }
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [False, False]
        mock_jload.return_value = manifest
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._get_image_attributes_v2_s1("/cont", manifest)
        self.assertEqual(status, (None, None))

        manifest = {
            "fsLayers": ({"blobSum": "foolayername"},),
            "history": ({"v1Compatibility": '["foojsonstring"]'},)
        }
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, True]
        mock_jload.return_value = manifest
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._get_image_attributes_v2_s1("/cont", manifest)
        self.assertEqual(status, (manifest, ['/cont/foolayername']))

    @patch('udocker.container.localrepo.json.loads')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_41__get_image_attr_v2_s2(self, mock_fu, mock_exists,
                                      mock_jload):
        """Test41 LocalRepository()._get_image_attributes_v2_s2()."""
        manifest = {
            "layers": ({"digest": "foolayername"},),
            "config": ({"digest": '["foojsonstring"]'},)
        }
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [False, False]
        mock_jload.return_value = manifest
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._get_image_attributes_v2_s2("/cont", manifest)
        self.assertEqual(status, (None, None))

        manifest = {
            "layers": ({"digest": "foolayername"},),
            "config": ({"digest": '["foojsonstring"]'},)
        }
        # res = (['foojsonstring'], ['/cont/foolayername'])
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, True]
        mock_jload.return_value = manifest
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._get_image_attributes_v2_s2("/cont", manifest)
        self.assertEqual(status, (None, ['/cont/foolayername']))

    @patch.object(LocalRepository, '_get_image_attributes_v2_s2')
    @patch.object(LocalRepository, '_get_image_attributes_v2_s1')
    @patch.object(LocalRepository, '_get_image_attributes_v1')
    @patch.object(LocalRepository, 'load_json')
    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_42_get_image_attributes(self, mock_fu, mock_exists,
                                     mock_loadjson, mock_attrv1,
                                     mock_attrv1s1, mock_attrv1s2):
        """Test42 LocalRepository().get_image_attributes()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [False, False]
        mock_loadjson.return_value = None
        mock_attrv1.return_value = (None, None)
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_image_attributes()
        self.assertEqual(status, (None, None))
        self.assertTrue(mock_exists.call_count, 2)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, False]
        mock_loadjson.return_value = None
        mock_attrv1.return_value = (['lname1'], ['/cont/lname1.layer'])
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_image_attributes()
        self.assertEqual(status, (['lname1'], ['/cont/lname1.layer']))
        self.assertTrue(mock_exists.call_count, 1)
        self.assertTrue(mock_attrv1.call_count, 1)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        manifest = {
            "fsLayers": ({"blobSum": "foolayername"},),
            "history": ({"v1Compatibility": '["foojsonstring"]'},)
        }
        mock_exists.side_effect = [False, True]
        mock_loadjson.return_value = manifest
        mock_attrv1s1.return_value = (['lname1'], ['/cont/lname1.layer'])
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_image_attributes()
        self.assertEqual(status, (['lname1'], ['/cont/lname1.layer']))
        self.assertTrue(mock_exists.call_count, 1)
        self.assertTrue(mock_attrv1s1.call_count, 1)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        manifest = {
            "layers": ({"digest": "foolayername"},),
            "config": ({"digest": '["foojsonstring"]'},)
        }
        mock_exists.side_effect = [False, True]
        mock_loadjson.return_value = manifest
        mock_attrv1s2.return_value = (['lname1'], ['/cont/lname1.layer'])
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo.get_image_attributes()
        self.assertEqual(status, (['lname1'], ['/cont/lname1.layer']))
        self.assertTrue(mock_exists.call_count, 1)
        self.assertTrue(mock_attrv1s2.call_count, 1)

    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_43_save_json(self, mock_fu, mock_exists):
        """Test43 LocalRepository().save_json()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, True]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = ""
        lrepo.cur_tagdir = ""
        status = lrepo.save_json("filename", "data")
        self.assertFalse(mock_exists.called)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [False, False]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = lrepo.reposdir + "/IMAGE"
        lrepo.cur_tagdir = lrepo.cur_repodir + "/TAG"
        status = lrepo.save_json("filename", "data")
        self.assertTrue(mock_exists.call_count, 1)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, False]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = lrepo.reposdir + "/IMAGE"
        lrepo.cur_tagdir = lrepo.cur_repodir + "/TAG"
        status = lrepo.save_json("filename", "data")
        self.assertTrue(mock_exists.call_count, 2)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, True]
        with patch(BOPEN, mock_open()) as mopen:
            lrepo = LocalRepository(UDOCKER_TOPDIR)
            lrepo.cur_repodir = lrepo.reposdir + "/IMAGE"
            lrepo.cur_tagdir = lrepo.cur_repodir + "/TAG"
            status = lrepo.save_json("filename", "data")
            self.assertTrue(mock_exists.call_count, 2)
            self.assertTrue(mopen.called)
            self.assertTrue(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, True]
        with patch(BOPEN, mock_open()) as mopen:
            mopen.side_effect = IOError('foo')
            lrepo = LocalRepository(UDOCKER_TOPDIR)
            status = lrepo.save_json("/filename", "data")
            self.assertTrue(mopen.called)
            self.assertFalse(status)

    @patch('udocker.container.localrepo.os.path.exists')
    @patch('udocker.container.localrepo.FileUtil')
    def test_44_load_json(self, mock_fu, mock_exists):
        """Test44 LocalRepository().load_json()."""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, True]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = ""
        lrepo.cur_tagdir = ""
        status = lrepo.load_json("filename")
        self.assertFalse(mock_exists.called)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [False, False]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = lrepo.reposdir + "/IMAGE"
        lrepo.cur_tagdir = lrepo.cur_repodir + "/TAG"
        status = lrepo.load_json("filename")
        self.assertTrue(mock_exists.call_count, 1)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, False]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        lrepo.cur_repodir = lrepo.reposdir + "/IMAGE"
        lrepo.cur_tagdir = lrepo.cur_repodir + "/TAG"
        status = lrepo.load_json("filename")
        self.assertTrue(mock_exists.call_count, 2)
        self.assertFalse(status)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, True]
        with patch(BOPEN, mock_open()) as mopen:
            lrepo = LocalRepository(UDOCKER_TOPDIR)
            lrepo.cur_repodir = lrepo.reposdir + "/IMAGE"
            lrepo.cur_tagdir = lrepo.cur_repodir + "/TAG"
            status = lrepo.load_json("filename")
            self.assertTrue(mock_exists.call_count, 2)
            self.assertTrue(mopen.called)
            self.assertEqual(status, None)

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_exists.side_effect = [True, True]
        with patch(BOPEN, mock_open()) as mopen:
            mopen.side_effect = IOError('foo')
            lrepo = LocalRepository(UDOCKER_TOPDIR)
            status = lrepo.load_json("/filename")
            self.assertTrue(mopen.called)
            self.assertFalse(status)

    @patch.object(LocalRepository, 'load_json')
    @patch('udocker.container.localrepo.os.listdir')
    @patch('udocker.container.localrepo.FileUtil')
    def test_45__load_structure(self, mock_fu, mock_listdir,
                                mock_json):
        """Test45 LocalRepository()._load_structure().
        Scan the repository structure of a given image tag.
        """
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        mock_fu.return_value.isdir.return_value = False
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._load_structure("IMAGETAGDIR")
        self.assertEqual(status, {"repolayers": dict()})

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        res = {'repolayers': {}, 'ancestry': {'layers': 'foolayername'}}
        mock_fu.return_value.isdir.return_value = True
        mock_listdir.return_value = ["ancestry"]
        jsload = {"layers": "foolayername"}
        mock_json.return_value = jsload
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._load_structure("IMAGETAGDIR")
        self.assertEqual(status, res)

    @patch('udocker.container.localrepo.FileUtil')
    def test_46__find_top_layer_id(self, mock_fu):
        """Test46 LocalRepository()._find_top_layer_id"""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        struct = ""
        lid = ""
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._find_top_layer_id(struct, lid)
        self.assertEqual(status, "")

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        struct = {'repolayers': {"lay1": "l1",
                                 "json": {"j1": 1, "parent": "123"}},
                  'ancestry': {'layers': 'foolayername'}}
        lid = "123"
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._find_top_layer_id(struct, lid)
        self.assertEqual(status, "123")

    # def test_47__sorted_layers(self):
    #     """Test47 LocalRepository()._sorted_layers"""
    #     pass

    @patch('udocker.container.localrepo.FileUtil')
    def test_48__split_layer_id(self, mock_fu):
        """Test48 LocalRepository()._split_layer_id"""
        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        lid = ("524b0c1e57f8ee5fee01a1decba2f30"
               "1c324a6513ca3551021264e3aa7341ebc")
        lidsha = "sha256:" + lid
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._split_layer_id(lidsha)
        self.assertEqual(status, ["sha256", lid])

        mock_fu.return_value.register_prefix.side_effect = \
            [None, None, None]
        lrepo = LocalRepository(UDOCKER_TOPDIR)
        status = lrepo._split_layer_id(lid)
        self.assertEqual(status, ("", lid))

    # def test_49__verify_layer_file(self):
    #     """Test49 LocalRepository()._verify_layer_file"""
    #     pass

    # def test_50__verify_image_v1(self):
    #     """Test50 LocalRepository()._verify_image_v1"""
    #     pass

    # def test_51__verify_image_v2_s1(self):
    #     """Test51 LocalRepository()._verify_image_v2_s1"""
    #     pass

    # def test_52__verify_image_v2_s2(self):
    #     """Test52 LocalRepository()._verify_image_v2_s2"""
    #     pass

    # def test_53_verify_image(self):
    #     """Test53 LocalRepository().verify_image"""
    #     pass


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""Manipulate volume bindings host_path:container_path"""


class Uvolume(object):
    """Manipulate volumes"""

    def __init__(self, volume=""):
        self.volume = volume

    def cleanpath(self, path):
        """Remove duplicate and trailing slashes"""
        clean_path = ""
        p_char = ''
        for char in str(path):
            if not clean_path:
                clean_path = char
            else:
                if not (char == p_char and char == '/'):
                    clean_path += char
            p_char = char

        if clean_path == "/":
            return clean_path

        return clean_path.rstrip('/')

    def split(self):
        """Split volume string host_path:container_path into list"""
        try:
            (host_dir, cont_dir) = self.volume.split(":", 1)
            if not cont_dir:
                cont_dir = host_dir
        except ValueError:
            host_dir = self.volume
            cont_dir = self.volume
        return (self.cleanpath(host_dir), self.cleanpath(cont_dir))



#!/usr/bin/env python

# -*- coding: utf-8 -*-
"""
udocker unit tests: ContainerStructure
"""

from unittest import TestCase, main
from unittest.mock import patch, Mock
from udocker.container.structure import ContainerStructure
from udocker.config import Config


class ContainerStructureTestCase(TestCase):
    """Test ContainerStructure() class for containers structure."""

    def setUp(self):
        Config().getconf()
        Config().conf['hostauth_list'] = ("/etc/passwd", "/etc/group")
        Config().conf['cmd'] = "/bin/bash"
        Config().conf['cpu_affinity_exec_tools'] = \
            (["numactl", "-C", "%s", "--", ],
             ["taskset", "-c", "%s", ])
        Config().conf['valid_host_env'] = "HOME"
        Config().conf['username'] = "user"
        Config().conf['userhome'] = "/"
        Config().conf['location'] = ""
        Config().conf['oskernel'] = "4.8.13"

        str_local = 'udocker.container.localrepo.LocalRepository'
        self.lrepo = patch(str_local)
        self.local = self.lrepo.start()
        self.mock_lrepo = Mock()
        self.local.return_value = self.mock_lrepo

    def tearDown(self):
        self.lrepo.stop()

    def test_01_init(self):
        """Test01 ContainerStructure()."""
        prex = ContainerStructure(self.local)
        self.assertEqual(prex.tag, "")
        self.assertEqual(prex.imagerepo, "")
        self.assertEqual(prex.localrepo, self.local)

        prex = ContainerStructure(self.local, "123456")
        self.assertEqual(prex.tag, "")
        self.assertEqual(prex.imagerepo, "")
        self.assertEqual(prex.container_id, "123456")

    @patch('udocker.container.structure.Msg')
    def test_02_get_container_attr(self, mock_msg):
        """Test02 ContainerStructure().get_container_attr()."""
        mock_msg.return_value.level.return_value = 0
        Config().conf['location'] = "/"
        prex = ContainerStructure(self.local)
        (container_dir, container_json) = prex.get_container_attr()
        self.assertEqual(container_dir, "")
        self.assertEqual(container_json, [])

        Config().conf['location'] = ""
        self.local.cd_container.return_value = ""
        prex = ContainerStructure(self.local)
        (container_dir, container_json) = prex.get_container_attr()
        self.assertEqual(container_dir, False)
        self.assertEqual(container_json, False)
        self.assertTrue(mock_msg.return_value.err.called)

        Config().conf['location'] = ""
        self.local.cd_container.return_value = "/"
        self.local.load_json.return_value = []
        prex = ContainerStructure(self.local)
        (container_dir, container_json) = prex.get_container_attr()
        self.assertEqual(container_dir, False)
        self.assertEqual(container_json, False)
        self.assertTrue(mock_msg.return_value.err.called)

        Config().conf['location'] = ""
        self.local.cd_container.return_value = "/"
        self.local.load_json.return_value = ["value", ]
        prex = ContainerStructure(self.local)
        (container_dir, container_json) = prex.get_container_attr()
        self.assertEqual(container_dir, "/")
        self.assertEqual(container_json, ["value", ])

    def test_03_get_container_meta(self):
        """Test03 ContainerStructure().get_container_meta()."""
        container_json = {
            "architecture": "amd64",
            "author": "https://github.com/CentOS/sig-cloud-instance-images",
            "config": {
                "AttachStderr": False,
                "AttachStdin": False,
                "AttachStdout": False,
                "Cmd": [
                    "/bin/bash"
                ],
                "Domainname": "",
                "Entrypoint": None,
                "Env": [
                    "PATH=\
                    /usr/local/sbin:\
                    /usr/local/bin:/usr/sbin:\
                    /usr/bin:/sbin:\
                    /bin"
                ],
                "Hostname": "9aac06993d69",
                "Image": "sha256:4f64745dd34556af8f644a7886fcf" +
                         "cb11c059f64e1b0a753cb41188656ec8b33",
                "Labels": {
                    "build-date": "20161102",
                    "license": "GPLv2",
                    "name": "CentOS Base Image",
                    "vendor": "CentOS"
                },
                "OnBuild": None,
                "OpenStdin": False,
                "StdinOnce": False,
                "Tty": False,
                "User": "",
                "Volumes": None,
                "WorkingDir": ""
            },
        }
        prex = ContainerStructure(self.local)
        status = prex.get_container_meta("Cmd", "", container_json)
        self.assertEqual(status, "/bin/bash")

        prex = ContainerStructure(self.local)
        status = prex.get_container_meta("XXX", "", container_json)
        self.assertEqual(status, "")

        prex = ContainerStructure(self.local)
        status = prex.get_container_meta("Entrypoint", "BBB", container_json)
        self.assertEqual(status, "BBB")

    def test_04__dict_to_str(self):
        """Test04 ContainerStructure()._dict_to_str()."""
        prex = ContainerStructure(self.local)
        status = prex._dict_to_str({'A': 1, 'B': 2})
        self.assertTrue(status in ("A:1 B:2 ", "B:2 A:1 ", ))
        self.assertEqual(status, "A:1 B:2 ")

    def test_05__dict_to_list(self):
        """Test05 ContainerStructure()._dict_to_list()."""
        prex = ContainerStructure(self.local)
        status = prex._dict_to_list({'A': 1, 'B': 2})
        self.assertEqual(sorted(status), sorted(["A:1", "B:2"]))

    @patch('udocker.container.structure.os.path.exists')
    def test_06__chk_container_root(self, mock_exists):
        """Test06 ContainerStructure()._chk_container_root()."""
        self.local.cd_container.return_value = ""
        prex = ContainerStructure(self.local)
        status = prex._chk_container_root("12345")
        self.assertEqual(status, 0)

        self.local.cd_container.return_value = "/ROOT"
        mock_exists.side_effect = [True, True, False, True,
                                   True, False, False, False,
                                   True, False, True, True, False]
        prex = ContainerStructure(self.local)
        status = prex._chk_container_root()
        self.assertEqual(status, 7)

    @patch.object(ContainerStructure, '_untar_layers')
    @patch('udocker.container.structure.Unique.uuid')
    @patch('udocker.container.structure.Msg')
    def test_07_create_fromimage(self, mock_msg, mock_uuid, mock_untar):
        """Test07 ContainerStructure().create_fromimage()."""
        mock_msg.return_value.level.return_value = 0
        self.local.cd_imagerepo.return_value = ""
        prex = ContainerStructure(self.local)
        status = prex.create_fromimage("imagerepo", "tag")
        self.assertFalse(status)

        self.local.cd_imagerepo.return_value = "/"
        self.local.get_image_attributes.return_value = ([], [])
        prex = ContainerStructure(self.local)
        status = prex.create_fromimage("imagerepo", "tag")
        self.assertFalse(status)

        self.local.cd_imagerepo.return_value = "/"
        self.local.get_image_attributes.return_value = (["value", ], [])
        self.local.setup_container.return_value = ""
        prex = ContainerStructure(self.local)
        status = prex.create_fromimage("imagerepo", "tag")
        self.assertFalse(status)

        self.local.cd_imagerepo.return_value = "/"
        self.local.get_image_attributes.return_value = (["value", ], [])
        self.local.setup_container.return_value = "/"
        mock_untar.return_value = False
        mock_uuid.return_value = "123456"
        prex = ContainerStructure(self.local)
        status = prex.create_fromimage("imagerepo", "tag")
        self.assertEqual(status, "123456")

    @patch.object(ContainerStructure, '_chk_container_root')
    @patch.object(ContainerStructure, '_untar_layers')
    @patch('udocker.container.structure.Unique.uuid')
    @patch('udocker.container.structure.Msg')
    def test_08_create_fromlayer(self, mock_msg, mock_uuid,
                                 mock_untar, mock_chkcont):
        """Test08 ContainerStructure().create_fromlayer()."""
        # Empty container_json
        cont_json = dict()
        mock_msg.return_value.level.return_value = 0
        mock_uuid.return_value = "123456"
        prex = ContainerStructure(self.local)
        status = prex.create_fromlayer("imagerepo", "tag", "layer", cont_json)
        self.assertFalse(status)

        # Non-empty container_json, empty cont dir
        cont_json = {
            "architecture": "amd64",
            "author": "https://github.com/CentOS/sig-cloud-instance-images",
            "config": {
                "AttachStderr": False,
                "AttachStdin": False,
                "AttachStdout": False,
                "Cmd": [
                    "/bin/bash"
                ],
                "Domainname": "",
                "Entrypoint": None,
                "Env": [
                    "PATH=\
                    /usr/local/sbin:\
                    /usr/local/bin:/usr/sbin:\
                    /usr/bin:/sbin:\
                    /bin"
                ],
                "Hostname": "9aac06993d69",
                "Image": "sha256:4f64745dd34556af8f644a7886fcf" +
                         "cb11c059f64e1b0a753cb41188656ec8b33",
                "Labels": {
                    "build-date": "20161102",
                    "license": "GPLv2",
                    "name": "CentOS Base Image",
                    "vendor": "CentOS"
                },
                "OnBuild": None,
                "OpenStdin": False,
                "StdinOnce": False,
                "Tty": False,
                "User": "",
                "Volumes": None,
                "WorkingDir": ""
            },
        }
        mock_msg.return_value.level.return_value = 0
        mock_uuid.return_value = "123456"
        self.local.setup_container.return_value = ""
        prex = ContainerStructure(self.local)
        status = prex.create_fromlayer("imagerepo", "tag", "layer", cont_json)
        self.assertFalse(status)

        # Non-empty container_json, non empty cont dir
        mock_msg.return_value.level.return_value = 0
        mock_uuid.return_value = "123456"
        self.local.setup_container.return_value = "/ROOT"
        self.local.save_json.return_value = True
        mock_untar.return_value = True
        mock_chkcont.return_value = 3
        prex = ContainerStructure(self.local)
        status = prex.create_fromlayer("imagerepo", "tag", "layer", cont_json)
        self.assertEqual(status, "123456")

    @patch.object(ContainerStructure, '_chk_container_root')
    @patch.object(ContainerStructure, '_untar_layers')
    @patch('udocker.container.structure.Unique.uuid')
    @patch('udocker.container.structure.Msg')
    def test_09_clone_fromfile(self, mock_msg, mock_uuid,
                               mock_untar, mock_chkcont):
        """Test09 ContainerStructure().clone_fromfile()."""
        # Empty container_dir
        mock_msg.return_value.level.return_value = 0
        self.local.setup_container.return_value = ""
        mock_uuid.return_value = "123456"
        prex = ContainerStructure(self.local)
        status = prex.clone_fromfile("clone_file")
        self.assertFalse(status)

        # Non-empty container_dir
        mock_msg.return_value.level.return_value = 0
        self.local.setup_container.return_value = "/ROOT"
        mock_uuid.return_value = "123456"
        mock_untar.return_value = True
        mock_chkcont.return_value = 3
        prex = ContainerStructure(self.local)
        status = prex.clone_fromfile("clone_file")
        self.assertEqual(status, "123456")

    @patch('udocker.container.structure.os.listdir')
    @patch('udocker.container.structure.os.path.isdir')
    @patch('udocker.container.structure.os.path.dirname')
    @patch('udocker.container.structure.os.path.basename')
    @patch('udocker.container.structure.Uprocess.get_output')
    @patch('udocker.container.structure.HostInfo.cmd_has_option')
    @patch('udocker.container.structure.FileUtil.remove')
    def test_10__apply_whiteouts(self, mock_furm, mock_hinfocmd,
                                 mock_uprocget, mock_base, mock_dir,
                                 mock_isdir, mock_lsdir):
        """Test10 ContainerStructure()._apply_whiteouts()."""
        mock_hinfocmd.return_value = False
        mock_uprocget.return_value = ""
        prex = ContainerStructure(self.local)
        prex._apply_whiteouts("tarball", "/tmp")
        self.assertTrue(mock_hinfocmd.called)
        self.assertTrue(mock_uprocget.called)

        mock_hinfocmd.return_value = False
        mock_uprocget.return_value = "/d1/.wh.aa\n/d1/.wh.bb\n"
        mock_base.side_effect = [".wh.aa", ".wh.bb", ".wh.aa", ".wh.bb"]
        mock_dir.side_effect = ["/d1", "/d2"]
        mock_isdir.side_effect = [True, True]
        mock_lsdir.side_effect = [".wh.aa", ".wh.bb"]
        mock_furm.side_effect = [None, None]
        prex = ContainerStructure(self.local)
        prex._apply_whiteouts("tarball", "/tmp")
        self.assertTrue(mock_hinfocmd.called)
        self.assertTrue(mock_uprocget.called)
        self.assertTrue(mock_base.called)
        self.assertTrue(mock_dir.called)
        self.assertTrue(mock_furm.called)

    @patch('udocker.container.structure.HostInfo')
    @patch('udocker.container.structure.subprocess.call')
    @patch.object(ContainerStructure, '_apply_whiteouts')
    @patch('udocker.container.structure.Msg')
    def test_11__untar_layers(self, mock_msg, mock_appwhite, mock_call,
                              mock_hinfo):
        """Test11 ContainerStructure()._untar_layers()."""
        mock_msg.level = 0
        tarfiles = ["a.tar", "b.tar", ]
        mock_msg.VER = 3
        mock_hinfo.gid = "1000"
        mock_hinfo.return_value.cmd_has_option.return_value = False
        mock_appwhite.side_effect = [None, None]
        mock_call.side_effect = [1, 1, 1, 1]
        prex = ContainerStructure(self.local)
        status = prex._untar_layers(tarfiles, "/tmp")
        self.assertFalse(status)
        self.assertTrue(mock_call.call_count, 2)
        self.assertTrue(mock_appwhite.call_count, 2)

        mock_msg.level = 0
        tarfiles = ["a.tar", "b.tar", ]
        mock_msg.VER = 3
        mock_hinfo.gid = "1000"
        mock_hinfo.return_value.cmd_has_option.return_value = False
        mock_appwhite.side_effect = [None, None]
        mock_call.side_effect = [0, 0, 0, 0]
        prex = ContainerStructure(self.local)
        status = prex._untar_layers(tarfiles, "/tmp")
        self.assertTrue(status)
        self.assertTrue(mock_call.call_count, 2)
        self.assertTrue(mock_appwhite.call_count, 2)

    @patch('udocker.container.structure.FileUtil.tar')
    @patch('udocker.container.structure.Msg')
    def test_12_export_tofile(self, mock_msg, mock_futar):
        """Test12 ContainerStructure().export_tofile()."""
        # Empty container dir
        mock_msg.return_value.level.return_value = 0
        self.local.cd_container.return_value = ""
        mock_futar.return_value = False
        prex = ContainerStructure(self.local)
        status = prex.export_tofile("clone_file")
        self.assertFalse(status)

        # Non-empty container dir
        mock_msg.return_value.level.return_value = 0
        self.local.cd_container.return_value = "/ROOT"
        mock_futar.return_value = True
        prex = ContainerStructure(self.local, "123456")
        status = prex.export_tofile("clone_file")
        self.assertEqual(status, "123456")

    @patch('udocker.container.structure.FileUtil.tar')
    @patch('udocker.container.structure.Msg')
    def test_13_clone_tofile(self, mock_msg, mock_futar):
        """Test13 ContainerStructure().clone_tofile()."""
        # Empty container dir
        mock_msg.return_value.level.return_value = 0
        self.local.cd_container.return_value = ""
        mock_futar.return_value = False
        prex = ContainerStructure(self.local)
        status = prex.clone_tofile("clone_file")
        self.assertFalse(status)

        # Non-empty container dir
        mock_msg.return_value.level.return_value = 0
        self.local.cd_container.return_value = "/ROOT"
        mock_futar.return_value = True
        prex = ContainerStructure(self.local, "123456")
        status = prex.clone_tofile("clone_file")
        self.assertEqual(status, "123456")

    @patch.object(ContainerStructure, '_chk_container_root')
    @patch('udocker.container.structure.FileUtil.copydir')
    @patch('udocker.container.structure.Unique.uuid')
    @patch('udocker.container.structure.Msg')
    def test_14_clone(self, mock_msg, mock_uuid,
                      mock_fucpd, mock_chkcont):
        """Test14 ContainerStructure().clone()."""
        # Empty source container_dir
        mock_msg.return_value.level.return_value = 0
        self.local.cd_container.return_value = ""
        mock_uuid.return_value = "123456"
        prex = ContainerStructure(self.local)
        status = prex.clone()
        self.assertFalse(status)

        # Non-empty source container_dir
        mock_msg.return_value.level.return_value = 0
        self.local.cd_container.return_value = "/ROOT/src"
        self.local.setup_container.return_value = "/ROOT/dst"
        mock_uuid.return_value = "123456"
        mock_chkcont.return_value = 3
        mock_fucpd.return_value = False
        prex = ContainerStructure(self.local)
        status = prex.clone()
        self.assertFalse(status)

        mock_msg.return_value.level.return_value = 0
        self.local.cd_container.return_value = "/ROOT/src"
        self.local.setup_container.return_value = "/ROOT/dst"
        mock_uuid.return_value = "123456"
        mock_chkcont.return_value = False
        mock_fucpd.return_value = True
        prex = ContainerStructure(self.local)
        status = prex.clone()
        self.assertEqual(status, "123456")


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
udocker unit tests: ExecutionMode
"""

from unittest import TestCase, main
from unittest.mock import Mock, patch
from udocker.engine.execmode import ExecutionMode
from udocker.engine.proot import PRootEngine
from udocker.engine.runc import RuncEngine
from udocker.engine.fakechroot import FakechrootEngine
from udocker.engine.singularity import SingularityEngine
from udocker.config import Config


class ExecutionModeTestCase(TestCase):
    """Test ExecutionMode()."""

    def setUp(self):
        Config().getconf()
        Config().conf['hostauth_list'] = ("/etc/passwd", "/etc/group")
        Config().conf['cmd'] = "/bin/bash"
        Config().conf['cpu_affinity_exec_tools'] = \
            (["numactl", "-C", "%s", "--", ], ["taskset", "-c", "%s", ])
        Config().conf['valid_host_env'] = "HOME"
        Config().conf['username'] = "user"
        Config().conf['userhome'] = "/"
        Config().conf['oskernel'] = "4.8.13"
        Config().conf['location'] = ""
        Config().conf['keystore'] = "KEYSTORE"
        Config().conf['osversion'] = "OSVERSION"
        Config().conf['arch'] = "ARCH"
        Config().conf['default_execution_mode'] = "P1"
        self.container_id = "CONTAINER_ID"

        str_local = 'udocker.container.localrepo.LocalRepository'
        self.lrepo = patch(str_local)
        self.local = self.lrepo.start()
        self.mock_lrepo = Mock()
        self.local.return_value = self.mock_lrepo

        str_cd = str_local + '.cd_container'
        self.cdcont = patch(str_cd, return_value="/tmp")
        self.mock_cdcont = self.cdcont.start()

    def tearDown(self):
        self.lrepo.stop()
        self.cdcont.stop()

    def test_01_init(self):
        """Test01 ExecutionMode() constructor."""
        uexm = ExecutionMode(self.local, self.container_id)
        self.assertEqual(uexm.localrepo, self.local)
        self.assertEqual(uexm.container_id, "CONTAINER_ID")
        self.assertEqual(uexm.container_dir, "/tmp")
        self.assertEqual(uexm.container_root, "/tmp/ROOT")
        self.assertEqual(uexm.container_execmode, "/tmp/execmode")
        self.assertIsNone(uexm.exec_engine)
        self.assertEqual(uexm.valid_modes,
                         ("P1", "P2", "F1", "F2",
                          "F3", "F4", "R1", "R2", "R3", "S1"))

    @patch('udocker.engine.execmode.FileUtil.getdata')
    def test_02_get_mode(self, mock_getdata):
        """Test02 ExecutionMode().get_mode."""
        mock_getdata.return_value.strip.return_value = None
        uexm = ExecutionMode(self.local, self.container_id)
        status = uexm.get_mode()
        self.assertEqual(status, "P1")

        mock_getdata.return_value = "F3"
        uexm = ExecutionMode(self.local, self.container_id)
        status = uexm.get_mode()
        self.assertEqual(status, "F3")

    @patch('udocker.engine.execmode.Msg')
    @patch('udocker.engine.execmode.os.path')
    @patch('udocker.engine.execmode.FileUtil.putdata')
    @patch('udocker.engine.execmode.FileBind.setup')
    @patch('udocker.engine.execmode.ElfPatcher.get_ld_libdirs')
    @patch('udocker.engine.execmode.FileUtil.links_conv')
    @patch('udocker.engine.execmode.FileBind.restore')
    @patch('udocker.engine.execmode.FileUtil.getdata')
    @patch('udocker.engine.execmode.FileUtil')
    @patch('udocker.engine.execmode.FileBind')
    @patch('udocker.engine.execmode.ElfPatcher')
    @patch.object(ExecutionMode, 'get_mode')
    def test_03_set_mode(self, mock_getmode, mock_elfp, mock_fbind,
                         mock_futil, mock_getdata, mock_restore, mock_links,
                         mock_getld, mock_fbset, mock_putdata, mock_path,
                         mock_msg):
        """Test03 ExecutionMode().set_mode."""
        mock_msg.level = 0
        mock_getmode.side_effect = \
            ["", "P1", "R1", "R1", "F4", "P1", "F3", "P2", "P2", "F4", "F4"]
        mock_getdata.return_value = "F3"
        mock_restore.return_value = None
        mock_fbset.return_value = None
        uexm = ExecutionMode(self.local, self.container_id)
        status = uexm.set_mode("")
        self.assertTrue(mock_getmode.called)
        self.assertTrue(mock_elfp.called)
        self.assertTrue(mock_fbind.called)
        self.assertTrue(mock_futil.called)
        self.assertFalse(status)

        mock_restore.return_value = None
        mock_fbset.return_value = None
        uexm = ExecutionMode(self.local, self.container_id)
        status = uexm.set_mode("P1")
        self.assertTrue(status)

        mock_restore.return_value = None
        mock_fbset.return_value = None
        mock_links.return_value = True
        mock_getld.return_value = True
        uexm = ExecutionMode(self.local, self.container_id)
        status = uexm.set_mode("F1", True)
        self.assertTrue(status)

        mock_restore.return_value = None
        mock_fbset.return_value = None
        mock_putdata.return_value = True
        mock_path.return_value = "/tmp"
        uexm = ExecutionMode(self.local, self.container_id)
        status = uexm.set_mode("P1")
        self.assertTrue(status)

        mock_restore.return_value = None
        mock_fbset.return_value = None
        uexm = ExecutionMode(self.local, self.container_id)
        uexm.set_mode("F1")
        self.assertTrue(mock_futil.return_value.links_conv.called)

        mock_restore.return_value = None
        mock_fbset.return_value = None
        uexm = ExecutionMode(self.local, self.container_id)
        uexm.set_mode("P2")
        self.assertTrue(mock_elfp.return_value.restore_ld.called)

        mock_restore.return_value = None
        mock_fbset.return_value = None
        uexm = ExecutionMode(self.local, self.container_id)
        uexm.set_mode("F2")
        self.assertTrue(mock_elfp.return_value.restore_binaries.called)

        mock_restore.return_value = None
        mock_fbset.return_value = None
        uexm = ExecutionMode(self.local, self.container_id)
        uexm.set_mode("F2")
        self.assertTrue(mock_elfp.return_value.patch_ld.called)

        mock_restore.return_value = None
        mock_fbset.return_value = None
        uexm = ExecutionMode(self.local, self.container_id)
        uexm.set_mode("F3")
        self.assertTrue(mock_elfp.return_value.patch_ld.called and
                        mock_elfp.return_value.patch_binaries.called)

        mock_restore.return_value = None
        mock_fbset.return_value = None
        uexm = ExecutionMode(self.local, self.container_id)
        status = uexm.set_mode("F3")
        self.assertTrue(status)

    @patch.object(ExecutionMode, 'get_mode')
    def test_04_get_engine(self, mock_getmode):
        """Test04 ExecutionMode().get_engine."""
        mock_getmode.side_effect = ["R1", "F2", "P2", "S1"]
        uexm = ExecutionMode(self.local, self.container_id)
        exec_engine = uexm.get_engine()
        self.assertIsInstance(exec_engine, RuncEngine)

        uexm = ExecutionMode(self.local, self.container_id)
        exec_engine = uexm.get_engine()
        self.assertIsInstance(exec_engine, FakechrootEngine)

        uexm = ExecutionMode(self.local, self.container_id)
        exec_engine = uexm.get_engine()
        self.assertIsInstance(exec_engine, PRootEngine)

        uexm = ExecutionMode(self.local, self.container_id)
        exec_engine = uexm.get_engine()
        self.assertIsInstance(exec_engine, SingularityEngine)


if __name__ == '__main__':
    main()



# -*- coding: utf-8 -*-
"""Basic unshare for udocker maintenance"""

import os
import ctypes
import subprocess

from udocker.msg import Msg
from udocker.helper.hostinfo import HostInfo
from udocker.helper.nixauth import NixAuthentication

class Unshare(object):
    """Place a process in a namespace"""

    CLONE_NEWNS = 0x20000
    CLONE_NEWUTS = 0x4000000
    CLONE_NEWIPC = 0x8000000
    CLONE_NEWUSER = 0x10000000
    CLONE_NEWPID = 0x20000000
    CLONE_NEWNET = 0x40000000

    def unshare(self, flags):
        """Python implementation of unshare"""
        try:
            _unshare = ctypes.CDLL("libc.so.6").unshare
        except OSError:
            Msg().err("Error: in unshare: mapping libc")
            return False

        _unshare.restype = ctypes.c_int
        _unshare.argtypes = (ctypes.c_int, )

        if _unshare(flags) == -1:
            Msg().err("Error: in unshare:", os.strerror(-1))
            return False
        return True

    def namespace_exec(self, method, flags=CLONE_NEWUSER):
        """Execute command in namespace"""
        (pread1, pwrite1) = os.pipe()
        (pread2, pwrite2) = os.pipe()
        cpid = os.fork()
        if cpid:
            os.close(pwrite1)
            os.read(pread1, 1)  # wait
            user = HostInfo().username()
            newidmap = ["newuidmap", str(cpid), "0", str(HostInfo.uid), "1"]
            for (subid, subcount) in NixAuthentication().user_in_subuid(user):
                newidmap.extend(["1", subid, subcount])

            subprocess.call(newidmap)
            newidmap = ["newgidmap", str(cpid), "0", str(HostInfo.uid), "1"]
            for (subid, subcount) in NixAuthentication().user_in_subgid(user):
                newidmap.extend(["1", subid, subcount])

            subprocess.call(newidmap)
            os.close(pwrite2)   # notify
            (dummy, status) = os.waitpid(cpid, 0)
            if status % 256:
                Msg().err("Error: namespace exec action failed")
                return False

            return True

        self.unshare(flags)
        os.close(pwrite2)
        os.close(pwrite1)   # notify
        os.read(pread2, 1)  # wait
        try:
            os.setgid(0)
            os.setuid(0)
            os.setgroups([0, 0, ])
        except OSError:
            Msg().err("Error: setting ids and groups")
            return False

        # pylint: disable=protected-access
        os._exit(int(method()))
        return True



#!/usr/bin/env python
"""
udocker unit tests: ExecutionEngineCommon
"""

from unittest import TestCase, main
from unittest.mock import Mock, patch
from udocker.engine.base import ExecutionEngineCommon
from udocker.utils.uenv import Uenv
from udocker.config import Config


class ExecutionEngineCommonTestCase(TestCase):
    """Test ExecutionEngineCommon().
    Parent class for containers execution.
    """

    def setUp(self):
        Config().getconf()
        Config().conf['hostauth_list'] = ("/etc/passwd", "/etc/group")
        Config().conf['cmd'] = "/bin/bash"
        Config().conf['cpu_affinity_exec_tools'] = \
            (["numactl", "-C", "%s", "--", ], ["taskset", "-c", "%s", ])
        Config().conf['location'] = ""
        Config().conf['uid'] = 1000
        Config().conf['sysdirs_list'] = ["/", ]
        Config().conf['root_path'] = "/usr/sbin:/sbin:/usr/bin:/bin"
        Config().conf['user_path'] = "/usr/bin:/bin:/usr/local/bin"
        str_local = 'udocker.container.localrepo.LocalRepository'
        self.lrepo = patch(str_local)
        self.local = self.lrepo.start()
        self.mock_lrepo = Mock()
        self.local.return_value = self.mock_lrepo

        str_exmode = 'udocker.engine.execmode.ExecutionMode'
        self.execmode = patch(str_exmode)
        self.xmode = self.execmode.start()
        self.mock_execmode = Mock()
        self.xmode.return_value = self.mock_execmode

    def tearDown(self):
        self.lrepo.stop()
        self.execmode.stop()

    def test_01_init(self):
        """Test01 ExecutionEngineCommon() constructor"""
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        self.assertEqual(ex_eng.container_id, "")
        self.assertEqual(ex_eng.container_root, "")
        self.assertEqual(ex_eng.container_names, [])
        self.assertEqual(ex_eng.localrepo, self.local)
        self.assertEqual(ex_eng.exec_mode, self.xmode)
        self.assertEqual(ex_eng.opt["nometa"], False)
        self.assertEqual(ex_eng.opt["nosysdirs"], False)
        self.assertEqual(ex_eng.opt["dri"], False)
        self.assertEqual(ex_eng.opt["bindhome"], False)
        self.assertEqual(ex_eng.opt["hostenv"], False)
        self.assertEqual(ex_eng.opt["hostauth"], False)
        self.assertEqual(ex_eng.opt["novol"], [])
        self.assertEqual(ex_eng.opt["vol"], [])
        self.assertEqual(ex_eng.opt["cpuset"], "")
        self.assertEqual(ex_eng.opt["user"], "")
        self.assertEqual(ex_eng.opt["cwd"], "")
        self.assertEqual(ex_eng.opt["entryp"], "")
        self.assertEqual(ex_eng.opt["hostname"], "")
        self.assertEqual(ex_eng.opt["domain"], "")
        self.assertEqual(ex_eng.opt["volfrom"], [])

    @patch('udocker.engine.base.HostInfo.cmd_has_option')
    def test_02__has_option(self, mock_hinfocmd):
        """Test02 ExecutionEngineCommon()._has_option()."""
        mock_hinfocmd.return_value = True
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._has_option("opt1")
        self.assertTrue(status)
        self.assertTrue(mock_hinfocmd.called)

    def test_03__get_portsmap(self):
        """Test03 ExecutionEngineCommon()._get_portsmap()."""
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["portsmap"] = ["1024:1024", "2048:2048"]
        status = ex_eng._get_portsmap()
        self.assertEqual(status, {1024: 1024, 2048: 2048})

    @patch('udocker.engine.base.Msg')
    @patch('udocker.engine.base.HostInfo')
    @patch.object(ExecutionEngineCommon, '_get_portsmap')
    def test_04__check_exposed_ports(self, mock_getports, mock_hinfo, mock_msg):
        """Test04 ExecutionEngineCommon()._check_exposed_ports()."""
        mock_msg.level = 0
        mock_getports.return_value = {22: 22, 2048: 2048}
        mock_hinfo.uid = 1000
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["portsexp"] = ("22", "2048/tcp")
        status = ex_eng._check_exposed_ports()
        self.assertFalse(status)

        mock_getports.return_value = {22: 22, 2048: 2048}
        mock_hinfo.uid = 0
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["portsexp"] = ("22", "2048/tcp")
        status = ex_eng._check_exposed_ports()
        self.assertTrue(status)

    @patch('udocker.engine.base.FileUtil.find_exec')
    def test_05__set_cpu_affinity(self, mock_findexec):
        """Test05 ExecutionEngineCommon()._set_cpu_affinity()."""
        mock_findexec.return_value = ""
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._set_cpu_affinity()
        self.assertEqual(status, [])

        mock_findexec.return_value = "taskset"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._set_cpu_affinity()
        self.assertEqual(status, [])

        mock_findexec.return_value = "numactl"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["cpuset"] = "1-2"
        status = ex_eng._set_cpu_affinity()
        self.assertEqual(status, ["numactl", "-C", "1-2", "--"])

    @patch('udocker.engine.base.MountPoint')
    @patch('udocker.engine.base.FileUtil.isdir')
    def test_06__create_mountpoint(self, mock_isdir, mock_mpoint):
        """Test06 ExecutionEngineCommon()._create_mountpoint()."""
        hpath = "/bin"
        cpath = "/ROOT/bin"
        mock_isdir.return_value = False
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._create_mountpoint(hpath, cpath, True)
        self.assertTrue(status)
        self.assertTrue(mock_isdir.called)

        hpath = "/bin"
        cpath = "/ROOT/bin"
        mock_isdir.return_value = True
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.mountp = mock_mpoint
        ex_eng.mountp.create.return_value = True
        ex_eng.mountp.save.return_value = None
        status = ex_eng._create_mountpoint(hpath, cpath, True)
        self.assertTrue(status)
        self.assertTrue(ex_eng.mountp.save.called)

        hpath = "/bin"
        cpath = "/ROOT/bin"
        mock_isdir.return_value = True
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.mountp = mock_mpoint
        ex_eng.mountp.create.return_value = False
        ex_eng.mountp.save.return_value = None
        status = ex_eng._create_mountpoint(hpath, cpath, True)
        self.assertFalse(status)

    @patch('udocker.engine.base.Msg')
    @patch.object(ExecutionEngineCommon, '_create_mountpoint')
    @patch('udocker.engine.base.os.path.exists')
    @patch('udocker.engine.base.Uvolume.split')
    def test_07__check_volumes(self, mock_uvolsplit, mock_exists,
                               mock_crmpoint, mock_msg):
        """Test07 ExecutionEngineCommon()._check_volumes()."""
        mock_msg.level = 0
        mock_uvolsplit.return_value = ("HOSTDIR", "CONTDIR")
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["vol"] = list()
        ex_eng.opt["vol"].append("HOSTDIR:CONTDIR")
        status = ex_eng._check_volumes()
        self.assertFalse(status)
        self.assertTrue(mock_uvolsplit.called)

        mock_uvolsplit.return_value = ("/HOSTDIR", "CONTDIR")
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["vol"] = list()
        ex_eng.opt["vol"].append("/HOSTDIR:CONTDIR")
        status = ex_eng._check_volumes()
        self.assertFalse(status)

        Config.conf['sysdirs_list'] = ["/HOSTDIR"]
        mock_exists.return_value = False
        mock_uvolsplit.return_value = ("/HOSTDIR", "/CONTDIR")
        mock_crmpoint.return_value = False
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["vol"] = list()
        ex_eng.opt["vol"].append("/HOSTDIR:/CONTDIR")
        status = ex_eng._check_volumes()
        self.assertTrue(status)
        self.assertEqual(ex_eng.opt["vol"], list())
        self.assertTrue(mock_exists.called)

        Config.conf['sysdirs_list'] = ["/sys"]
        mock_exists.return_value = False
        mock_uvolsplit.return_value = ("/HOSTDIR", "/CONTDIR")
        mock_crmpoint.return_value = False
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["vol"] = list()
        ex_eng.opt["vol"].append("/HOSTDIR:/CONTDIR")
        status = ex_eng._check_volumes()
        self.assertFalse(status)
        self.assertTrue(mock_exists.called)

        Config.conf['sysdirs_list'] = ["/sys"]
        mock_exists.return_value = True
        mock_uvolsplit.return_value = ("/HOSTDIR", "/CONTDIR")
        mock_crmpoint.return_value = True
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["vol"] = list()
        ex_eng.opt["vol"].append("/HOSTDIR:/CONTDIR")
        status = ex_eng._check_volumes()
        self.assertTrue(status)

    @patch('udocker.engine.base.NixAuthentication.get_home')
    def test_08__get_bindhome(self, mock_gethome):
        """Test08 ExecutionEngineCommon()._get_bindhome()."""
        mock_gethome.return_value = ""
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["bindhome"] = False
        status = ex_eng._get_bindhome()
        self.assertEqual(status, "")

        mock_gethome.return_value = "/home/user"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["bindhome"] = True
        status = ex_eng._get_bindhome()
        self.assertEqual(status, "/home/user")
        self.assertTrue(mock_gethome.called)

    @patch('udocker.engine.base.Uvolume.cleanpath')
    @patch('udocker.engine.base.Uvolume.split')
    def test_09__is_volume(self, mock_uvolsplit, mock_uvolclean):
        """Test09 ExecutionEngineCommon()._is_volume()."""
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["vol"] = list()
        status = ex_eng._is_volume("/tmp")
        self.assertEqual(status, "")

        mock_uvolsplit.return_value = ("/tmp", "/CONTDIR")
        mock_uvolclean.return_value = "/tmp"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["vol"] = list()
        ex_eng.opt["vol"].append("/tmp:/CONTDIR")
        status = ex_eng._is_volume("/tmp")
        self.assertEqual(status, "/CONTDIR")

    @patch('udocker.engine.base.Uvolume.cleanpath')
    @patch('udocker.engine.base.Uvolume.split')
    def test_10__is_mountpoint(self, mock_uvolsplit, mock_uvolclean):
        """Test10 ExecutionEngineCommon()._is_mountpoint()."""
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["vol"] = list()
        status = ex_eng._is_mountpoint("/tmp")
        self.assertEqual(status, "")

        mock_uvolsplit.return_value = ("/tmp", "/CONTDIR")
        mock_uvolclean.return_value = "/CONTDIR"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["vol"] = list()
        ex_eng.opt["vol"].append("/tmp:/CONTDIR")
        status = ex_eng._is_mountpoint("/tmp")
        self.assertEqual(status, "/tmp")

    @patch.object(ExecutionEngineCommon, '_check_volumes')
    @patch.object(ExecutionEngineCommon, '_get_bindhome')
    def test_11__set_volume_bindings(self, mock_bindhome, mock_chkvol):
        """Test11 ExecutionEngineCommon()._set_volume_bindings()."""
        Config.conf['sysdirs_list'] = ["/sys"]
        Config.conf['dri_list'] = ["/dri"]
        mock_bindhome.return_value = "/home/user"
        mock_chkvol.return_value = False
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["vol"] = list()
        ex_eng.opt["hostauth"] = "/etc/passwd"
        ex_eng.opt["nosysdirs"] = False
        ex_eng.opt["dri"] = True
        ex_eng.opt["novol"] = list()
        status = ex_eng._set_volume_bindings()
        self.assertFalse(status)
        self.assertTrue(mock_bindhome.called)
        self.assertTrue(mock_chkvol.called)

        Config.conf['sysdirs_list'] = ["/sys"]
        Config.conf['dri_list'] = ["/dri"]
        mock_bindhome.return_value = "/home/user"
        mock_chkvol.return_value = True
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["vol"] = list()
        ex_eng.opt["hostauth"] = "/etc/passwd"
        ex_eng.opt["nosysdirs"] = False
        ex_eng.opt["dri"] = True
        ex_eng.opt["novol"] = ["/sys", "/dri", "/home/user"]
        status = ex_eng._set_volume_bindings()
        self.assertTrue(status)

    @patch('udocker.engine.base.Msg')
    @patch('udocker.engine.base.Uenv')
    @patch('udocker.engine.base.os.path.isdir')
    @patch('udocker.engine.base.FileUtil.cont2host')
    def test_12__check_paths(self, mock_fuc2h, mock_isdir, mock_uenv,
                             mock_msg):
        """Test12 ExecutionEngineCommon()._check_paths()."""
        Config.conf['root_path'] = "/sbin"
        Config.conf['user_path'] = "/bin"
        mock_msg.level = 0
        mock_fuc2h.return_value = "/container/bin"
        mock_isdir.return_value = False
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["uid"] = "0"
        ex_eng.opt["cwd"] = ""
        ex_eng.opt["home"] = "/home/user"
        ex_eng.opt["env"] = mock_uenv
        ex_eng.opt["env"].getenv.return_value = "/sbin"
        status = ex_eng._check_paths()
        self.assertFalse(status)
        self.assertEqual(ex_eng.opt["cwd"], ex_eng.opt["home"])

        Config.conf['root_path'] = "/sbin"
        Config.conf['user_path'] = "/bin"
        mock_fuc2h.return_value = "/container/bin"
        mock_isdir.return_value = True
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["uid"] = "0"
        ex_eng.opt["cwd"] = "/home/user"
        ex_eng.opt["home"] = "/home/user"
        ex_eng.opt["env"] = mock_uenv
        ex_eng.opt["env"].getenv.return_value = ""
        status = ex_eng._check_paths()
        self.assertTrue(status)
        self.assertTrue(mock_fuc2h.called)
        self.assertTrue(mock_isdir.called)

    @patch('udocker.engine.base.Msg')
    @patch('udocker.engine.base.FileUtil.find_exec')
    @patch('udocker.engine.base.Uenv')
    def test_13__check_executable(self, mock_uenv, mock_fufindexe, mock_msg):
        """Test13 ExecutionEngineCommon()._check_executable()."""
        Config.conf['cmd'] = "/bin/ls"
        mock_msg.level = 0
        mock_fufindexe.return_value = ""
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["env"] = mock_uenv
        ex_eng.opt["env"].getenv.return_value = ""
        ex_eng.opt["entryp"] = "/bin/ls -a -l"
        ex_eng.container_root = "/containers/123/ROOT"
        status = ex_eng._check_executable()
        self.assertEqual(status, "")
        self.assertTrue(mock_fufindexe.called)

        Config.conf['cmd'] = "/bin/ls"
        mock_fufindexe.return_value = "/bin/ls"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["env"] = mock_uenv
        ex_eng.opt["env"].getenv.return_value = ""
        ex_eng.opt["entryp"] = "/bin/ls -a -l"
        ex_eng.container_root = "/containers/123/ROOT"
        status = ex_eng._check_executable()
        self.assertEqual(status, "/containers/123/ROOT//bin/ls")

        Config.conf['cmd'] = "/bin/ls"
        mock_fufindexe.return_value = "/bin/ls"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["env"] = mock_uenv
        ex_eng.opt["env"].getenv.return_value = ""
        ex_eng.opt["entryp"] = ["/bin/ls", "-a", "-l"]
        ex_eng.container_root = "/containers/123/ROOT"
        status = ex_eng._check_executable()
        self.assertEqual(status, "/containers/123/ROOT//bin/ls")

    @patch('udocker.engine.base.ContainerStructure.get_container_meta')
    @patch('udocker.engine.base.ContainerStructure.get_container_attr')
    def test_14__run_load_metadata(self, mock_attr, mock_meta):
        """Test14 ExecutionEngineCommon()._run_load_metadata()."""
        Config().conf['location'] = "/tmp/container"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._run_load_metadata("123")
        self.assertEqual(status, ("", []))

        Config().conf['location'] = ""
        mock_attr.return_value = (None, None)
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._run_load_metadata("123")
        self.assertEqual(status, (None, None))
        self.assertTrue(mock_attr.called)

        Config().conf['location'] = ""
        mock_attr.return_value = ("/x", [])
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["nometa"] = True
        status = ex_eng._run_load_metadata("123")
        self.assertEqual(status, ("/x", []))

        Config().conf['location'] = ""
        mock_attr.return_value = ("/x", [])
        mock_meta.side_effects = ["user1", "cont/ROOT", "host1", "mydomain",
                                  "ls", "/bin/sh", "vol1", "8443", None]
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["nometa"] = False
        ex_eng.opt["portsexp"] = list()
        status = ex_eng._run_load_metadata("123")
        self.assertEqual(status, ("/x", []))
        self.assertTrue(mock_meta.call_count, 9)

    @patch('udocker.engine.base.os.path.isfile')
    @patch('udocker.engine.base.os.path.islink')
    @patch('udocker.engine.base.FileBind')
    @patch.object(ExecutionEngineCommon, '_is_mountpoint')
    def test_15__select_auth_files(self, mock_ismpoint, mock_fbind, mock_islink,
                                   mock_isfile):
        """Test15 ExecutionEngineCommon()._select_auth_files()."""
        resp = "/fbdir/#etc#passwd"
        resg = "/fbdir/#etc#group"
        mock_fbind.return_value.orig_dir = "/fbdir"
        mock_islink.side_effect = [True, True]
        mock_isfile.side_effect = [True, True]
        mock_ismpoint.side_effect = ["", "", "", ""]
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._select_auth_files()
        self.assertTrue(mock_islink.call_count, 2)
        self.assertTrue(mock_isfile.call_count, 2)
        self.assertTrue(mock_ismpoint.call_count, 2)
        # self.assertEqual(status, resp)

        resp = "/etc/passwd"
        resg = "/etc/group"
        mock_fbind.orig_dir.side_effect = ["/fbdir", "/fbdir"]
        mock_islink.side_effect = [False, False]
        mock_isfile.side_effect = [False, False]
        mock_ismpoint.side_effect = ["", "", "", ""]
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._select_auth_files()
        self.assertEqual(status, (resp, resg))

        resp = "/d/etc/passwd"
        resg = "/d/etc/group"
        mock_fbind.orig_dir.side_effect = ["/fbdir", "/fbdir"]
        mock_islink.side_effect = [False, False]
        mock_isfile.side_effect = [False, False]
        mock_ismpoint.side_effect = ["/d/etc/passwd", "/d/etc/passwd",
                                     "/d/etc/group", "/d/etc/group"]
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._select_auth_files()
        self.assertTrue(mock_ismpoint.call_count, 4)
        self.assertEqual(status, (resp, resg))

    def test_16__validate_user_str(self):
        """Test16 ExecutionEngineCommon()._validate_user_str()."""
        userstr = ""
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._validate_user_str(userstr)
        self.assertEqual(status, dict())

        userstr = "user1"
        res = {"user": userstr}
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._validate_user_str(userstr)
        self.assertEqual(status, res)

        userstr = "1000:1000"
        res = {"uid": "1000", "gid": "1000"}
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._validate_user_str(userstr)
        self.assertEqual(status, res)

    def test_17__user_from_str(self):
        """Test17 ExecutionEngineCommon()._user_from_str()."""
        userstr = ""
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._user_from_str(userstr)
        self.assertEqual(status, (False, dict()))

        userstr = "user1"
        res = {"user": userstr}
        str_exmode = 'udocker.helper.nixauth.NixAuthentication'
        nixauth = patch(str_exmode)
        auth = nixauth.start()
        mock_auth = Mock()
        auth.return_value = mock_auth
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["hostauth"] = True
        auth.get_user.return_value = ("user1", "1000", "1000", "usr",
                                      "/home/user", "/bin/bash")
        status = ex_eng._user_from_str(userstr, host_auth=auth)
        self.assertEqual(status, (True, res))
        auth = nixauth.stop()

        userstr = "user1"
        res = {"user": userstr}
        str_exmode = 'udocker.helper.nixauth.NixAuthentication'
        nixauth = patch(str_exmode)
        auth = nixauth.start()
        mock_auth = Mock()
        auth.return_value = mock_auth
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["hostauth"] = False
        auth.get_user.return_value = ("user1", "1000", "1000", "usr",
                                      "/home/user", "/bin/bash")
        status = ex_eng._user_from_str(userstr, container_auth=auth)
        self.assertEqual(status, (True, res))
        auth = nixauth.stop()

        userstr = "1000:1000"
        res = {"uid": "1000", "gid": "1000"}
        str_exmode = 'udocker.helper.nixauth.NixAuthentication'
        nixauth = patch(str_exmode)
        auth = nixauth.start()
        mock_auth = Mock()
        auth.return_value = mock_auth
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["hostauth"] = True
        auth.get_user.return_value = ("user1", "1000", "1000", "usr",
                                      "/home/user", "/bin/bash")
        status = ex_eng._user_from_str(userstr, host_auth=auth)
        self.assertEqual(status, (True, res))
        auth = nixauth.stop()

    @patch('udocker.engine.base.Msg')
    @patch('udocker.engine.base.HostInfo')
    @patch('udocker.engine.base.NixAuthentication')
    @patch.object(ExecutionEngineCommon, '_create_user')
    @patch.object(ExecutionEngineCommon, '_is_mountpoint')
    @patch.object(ExecutionEngineCommon, '_user_from_str')
    @patch.object(ExecutionEngineCommon, '_select_auth_files')
    def test_18__setup_container_user(self, mock_selauth, mock_ustr,
                                      mock_ismpoint, mock_cruser, mock_auth,
                                      mock_hinfo, mock_msg):
        """Test18 ExecutionEngineCommon()._setup_container_user()."""
        user = "invuser"
        mock_msg.level = 0
        mock_selauth.return_value = ("/etc/passwd", "/etc/group")
        mock_ustr.return_value = (False, {"uid": "100", "gid": "50"})
        mock_auth.side_effect = [None, None]
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._setup_container_user(user)
        self.assertFalse(status)

        user = ""
        mock_selauth.return_value = ("/etc/passwd", "/etc/group")
        mock_ustr.return_value = (True, {"uid": "0", "gid": "0"})
        mock_auth.side_effect = [None, None]
        mock_hinfo.return_value.username.return_value = "root"
        mock_hinfo.uid = 0
        mock_hinfo.gid = 0
        mock_ismpoint.side_effect = [True, True]
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._setup_container_user(user)
        self.assertTrue(status)

        user = ""
        mock_selauth.return_value = ("/etc/passwd", "/etc/group")
        mock_ustr.return_value = (True, {"uid": "0", "gid": "0"})
        mock_auth.side_effect = [None, None]
        mock_hinfo.return_value.username.return_value = "root"
        mock_hinfo.uid = 0
        mock_hinfo.gid = 0
        mock_ismpoint.side_effect = [False, False]
        mock_cruser.return_value = None
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["user"] = ""
        ex_eng.opt["hostauth"] = True
        status = ex_eng._setup_container_user(user)
        self.assertFalse(status)
        self.assertFalse(mock_cruser.called)

        user = ""
        mock_selauth.return_value = ("/etc/passwd", "/etc/group")
        mock_ustr.return_value = (True, {"uid": "0", "gid": "0"})
        mock_auth.side_effect = [None, None]
        mock_hinfo.return_value.username.return_value = "root"
        mock_hinfo.uid = 0
        mock_hinfo.gid = 0
        mock_ismpoint.side_effect = [False, False]
        mock_cruser.return_value = None
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["user"] = ""
        ex_eng.opt["hostauth"] = False
        ex_eng.opt["containerauth"] = False
        status = ex_eng._setup_container_user(user)
        self.assertTrue(status)
        self.assertTrue(mock_cruser.called)

    @patch('udocker.engine.base.Msg')
    @patch('udocker.engine.base.HostInfo')
    @patch('udocker.engine.base.NixAuthentication')
    @patch.object(ExecutionEngineCommon, '_create_user')
    @patch.object(ExecutionEngineCommon, '_is_mountpoint')
    @patch.object(ExecutionEngineCommon, '_user_from_str')
    @patch.object(ExecutionEngineCommon, '_select_auth_files')
    def test_19__set_cont_user_noroot(self, mock_selauth, mock_ustr,
                                      mock_ismpoint, mock_cruser, mock_auth,
                                      mock_hinfo, mock_msg):
        """Test19 ExecutionEngineCommon()._setup_container_user_noroot()."""
        user = "invuser"
        mock_msg.level = 0
        mock_selauth.return_value = ("/etc/passwd", "/etc/group")
        mock_ustr.return_value = (False, {"uid": "100", "gid": "50"})
        mock_auth.side_effect = [None, None]
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._setup_container_user_noroot(user)
        self.assertFalse(status)

        user = ""
        mock_selauth.return_value = ("/etc/passwd", "/etc/group")
        mock_ustr.return_value = (True, {"uid": "1000", "gid": "1000"})
        mock_auth.side_effect = [None, None]
        mock_hinfo.return_value.username.return_value = "u1"
        mock_hinfo.uid = 0
        mock_hinfo.gid = 0
        mock_ismpoint.side_effect = [True, True]
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._setup_container_user_noroot(user)
        self.assertTrue(status)

        user = ""
        mock_selauth.return_value = ("/etc/passwd", "/etc/group")
        mock_ustr.return_value = (True, {"uid": "1000", "gid": "1000"})
        mock_auth.side_effect = [None, None]
        mock_hinfo.return_value.username.return_value = "u1"
        mock_hinfo.uid = 0
        mock_hinfo.gid = 0
        mock_ismpoint.side_effect = [False, False]
        mock_cruser.return_value = None
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["user"] = ""
        ex_eng.opt["hostauth"] = True
        status = ex_eng._setup_container_user_noroot(user)
        self.assertFalse(status)
        self.assertFalse(mock_cruser.called)

        user = ""
        mock_selauth.return_value = ("/etc/passwd", "/etc/group")
        mock_ustr.return_value = (True, {"uid": "1000", "gid": "1000"})
        mock_auth.side_effect = [None, None]
        mock_hinfo.return_value.username.return_value = "u1"
        mock_hinfo.uid = 0
        mock_hinfo.gid = 0
        mock_ismpoint.side_effect = [False, False]
        mock_cruser.return_value = None
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["user"] = ""
        ex_eng.opt["hostauth"] = False
        ex_eng.opt["containerauth"] = False
        status = ex_eng._setup_container_user_noroot(user)
        self.assertTrue(status)
        self.assertTrue(mock_cruser.called)

    @patch('udocker.engine.base.HostInfo')
    @patch('udocker.engine.base.NixAuthentication')
    def test_20__fill_user(self, mock_auth, mock_hinfo):
        """Test20 ExecutionEngineCommon()._fill_user()."""
        mock_auth.return_value.get_home.return_value = "/home/u1"
        mock_hinfo.uid = 1000
        mock_hinfo.gid = 1000
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["user"] = ""
        ex_eng.opt["uid"] = ""
        ex_eng.opt["gid"] = ""
        ex_eng.opt["bindhome"] = True
        ex_eng.opt["shell"] = ""
        ex_eng.opt["gecos"] = ""
        ex_eng._fill_user()
        self.assertEqual(ex_eng.opt["user"], "udoc1000")
        self.assertEqual(ex_eng.opt["uid"], "1000")
        self.assertEqual(ex_eng.opt["gid"], "1000")
        self.assertEqual(ex_eng.opt["shell"], "/bin/sh")
        self.assertEqual(ex_eng.opt["gecos"], "*UDOCKER*")

    @patch.object(ExecutionEngineCommon, '_fill_user')
    @patch('udocker.engine.base.os.getgroups')
    @patch('udocker.engine.base.FileUtil.copyto')
    @patch('udocker.engine.base.FileUtil.mktmp')
    @patch('udocker.engine.base.FileUtil.umask')
    @patch('udocker.engine.base.NixAuthentication')
    def test_21__create_user(self, mock_nix, mock_umask, mock_mktmp,
                             mock_cpto, mock_getgrp, mock_fillu):
        """Test21 ExecutionEngineCommon()._create_user()."""
        cont_auth = mock_nix
        host_auth = mock_nix
        cont_auth.passwd_file.return_value = "/c/etc/passwd"
        cont_auth.group_file.return_value = "/c/etc/grpup"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["containerauth"] = True
        status = ex_eng._create_user(cont_auth, host_auth)
        self.assertTrue(status)
        self.assertFalse(ex_eng.opt["hostauth"])

        cont_auth = mock_nix
        host_auth = mock_nix
        res = ("/tmp/passwd:/etc/passwd","/tmp/group:/etc/group")
        cont_auth.passwd_file.return_value = "/c/etc/passwd"
        cont_auth.group_file.return_value = "/c/etc/group"
        mock_umask.side_effect = [None, None]
        mock_mktmp.side_effect = ["/tmp/passwd", "/tmp/group"]
        mock_cpto.side_effect = [None, None]
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["containerauth"] = False
        ex_eng.opt["hostauth"] = True
        status = ex_eng._create_user(cont_auth, host_auth)
        self.assertTrue(status)
        self.assertTrue(ex_eng.opt["hostauth"])
        self.assertEqual(ex_eng.hostauth_list, res)

        cont_auth = mock_nix
        host_auth = mock_nix
        res = ("/tmp/passwd:/etc/passwd","/tmp/group:/etc/group")
        cont_auth.passwd_file.return_value = "/c/etc/passwd"
        cont_auth.group_file.return_value = "/c/etc/group"
        mock_umask.side_effect = [None, None]
        mock_mktmp.side_effect = ["/tmp/passwd", "/tmp/group"]
        mock_cpto.side_effect = [None, None]
        mock_fillu.return_value = None
        mock_nix.add_user.return_value = None
        host_auth.get_group.return_value = ("", "x1", "x2")
        mock_nix.add_group.return_value = None
        mock_getgrp.return_value = [1000]
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["containerauth"] = False
        ex_eng.opt["hostauth"] = False
        status = ex_eng._create_user(cont_auth, host_auth)
        self.assertTrue(status)
        self.assertTrue(ex_eng.opt["hostauth"])
        self.assertEqual(ex_eng.hostauth_list, res)

    @patch('udocker.engine.base.Msg')
    @patch('udocker.engine.base.os.path.basename')
    def test_22__run_banner(self, mock_base, mock_msg):
        """Test22 ExecutionEngineCommon()._run_banner()."""
        mock_base.return_value = "ls"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng._run_banner("/bin/bash")
        ex_eng.container_id = "CONTAINERID"
        self.assertTrue(mock_msg.called)
        self.assertTrue(mock_base.called_once_with("/bin/bash"))

    @patch('udocker.engine.base.os.environ.copy')
    @patch('udocker.engine.base.os.environ')
    def test_23__run_env_cleanup_dict(self, mock_osenv, mock_osenvcp):
        """Test23 ExecutionEngineCommon()._run_env_cleanup_dict()."""
        res = {'HOME': '/', }
        Config.conf['valid_host_env'] = ("HOME",)
        Config.conf['invalid_host_env'] = ("USERNAME",)
        mock_osenvcp.return_value = {'HOME': '/', 'USERNAME': 'user', }
        mock_osenv.return_value = {'HOME': '/', }
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["hostenv"] = {'HOME': '/', }
        ex_eng._run_env_cleanup_dict()
        self.assertEqual(mock_osenv.return_value, res)

    @patch('udocker.engine.base.os.environ')
    def test_24__run_env_cleanup_list(self, mock_osenv):
        """Test24 ExecutionEngineCommon()._run_env_cleanup_list()."""
        Config.conf['valid_host_env'] = ("HOME",)
        Config.conf['invalid_host_env'] = ("USERNAME",)
        mock_osenv.return_value = {'HOME': '/', 'USERNAME': 'user', }
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["hostenv"] = "HOME"
        ex_eng.opt["env"] = Uenv()
        ex_eng._run_env_cleanup_list()
        self.assertEqual(ex_eng.opt["env"].env, dict())

    @patch('udocker.engine.base.HostInfo.username')
    def test_25__run_env_set(self, mock_hiuname):
        """Test25 ExecutionEngineCommon()._run_env_set()."""
        mock_hiuname.return_value = "user1"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["home"] = "/"
        ex_eng.opt["user"] = "user1"
        ex_eng.opt["uid"] = "1000"
        ex_eng.container_root = "/croot"
        ex_eng.container_id = "2717add4-e6f6-397c-9019-74fa67be439f"
        ex_eng.container_names = ['cna[]me', ]
        self.xmode.get_mode.return_value = "P1"
        ex_eng._run_env_set()
        self.assertEqual(ex_eng.opt["env"].env["USER"], "user1")
        self.assertEqual(ex_eng.opt["env"].env["LOGNAME"], "user1")
        self.assertEqual(ex_eng.opt["env"].env["USERNAME"], "user1")
        self.assertEqual(ex_eng.opt["env"].env["SHLVL"], "0")

    @patch('udocker.engine.base.FileUtil.getdata')
    def test_26__run_env_cmdoptions(self, mock_getdata):
        """Test26 ExecutionEngineCommon()._run_env_cmdoptions()."""
        mock_getdata.return_value = "USER=user1\nSHLVL=0"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        ex_eng.opt["envfile"] = ["/dir/env"]
        ex_eng.opt["env"] = Uenv()
        ex_eng._run_env_cmdoptions()
        self.assertEqual(ex_eng.opt["env"].env["USER"], "user1")

    @patch('udocker.engine.base.MountPoint')
    @patch.object(ExecutionEngineCommon, '_check_exposed_ports')
    @patch.object(ExecutionEngineCommon, '_set_volume_bindings')
    @patch.object(ExecutionEngineCommon, '_check_executable')
    @patch.object(ExecutionEngineCommon, '_check_paths')
    @patch.object(ExecutionEngineCommon, '_setup_container_user')
    @patch.object(ExecutionEngineCommon, '_run_env_cmdoptions')
    @patch.object(ExecutionEngineCommon, '_run_load_metadata')
    def test_27__run_init(self, mock_loadmeta, mock_envcmd,
                          mock_setupuser,
                          mock_chkpaths, mock_chkexec, mock_setvol,
                          mock_chkports, mock_mpoint):
        """Test27 ExecutionEngineCommon()._run_init()."""
        # mock_getcname.return_value = "cname"
        Config.conf['location'] = ""
        mock_loadmeta.return_value = ("", "dummy",)
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._run_init("2717add4-e6f6-397c-9019-74fa67be439f")
        self.assertEqual(status, "")
        self.assertTrue(mock_loadmeta.called)

        Config.conf['location'] = "/container_dir"
        mock_loadmeta.return_value = ("/container_dir", "dummy",)
        mock_envcmd.return_value = None
        mock_chkports.return_value = None
        mock_setupuser.return_value = False
        self.local.get_container_name.return_value = "cont_name"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._run_init("2717add4-e6f6-397c-9019-74fa67be439f")
        self.assertEqual(status, "")
        self.assertTrue(mock_envcmd.called)
        self.assertTrue(self.local.get_container_name.called)
        self.assertTrue(mock_chkports.called)

        Config.conf['location'] = "/container_dir"
        mock_loadmeta.return_value = ("/container_dir", "dummy",)
        mock_envcmd.return_value = None
        mock_chkports.return_value = None
        mock_setupuser.return_value = True
        mock_mpoint.return_value = None
        mock_setvol.return_value = False
        self.local.get_container_name.return_value = "cont_name"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._run_init("2717add4-e6f6-397c-9019-74fa67be439f")
        self.assertEqual(status, "")
        self.assertTrue(mock_chkports.called)
        self.assertTrue(mock_mpoint.called)
        self.assertTrue(mock_setvol.called)

        Config.conf['location'] = "/container_dir"
        mock_loadmeta.return_value = ("/container_dir", "dummy",)
        mock_envcmd.return_value = None
        mock_chkports.return_value = None
        mock_setupuser.return_value = True
        mock_mpoint.return_value = None
        mock_setvol.return_value = True
        mock_chkpaths.return_value = False
        self.local.get_container_name.return_value = "cont_name"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._run_init("2717add4-e6f6-397c-9019-74fa67be439f")
        self.assertEqual(status, "")
        self.assertTrue(mock_chkpaths.called)

        Config.conf['location'] = "/container_dir"
        mock_loadmeta.return_value = ("/container_dir", "dummy",)
        mock_envcmd.return_value = None
        mock_chkports.return_value = None
        mock_setupuser.return_value = True
        mock_mpoint.return_value = None
        mock_setvol.return_value = True
        mock_chkpaths.return_value = True
        mock_chkexec.return_value = "/bin/ls"
        self.local.get_container_name.return_value = "cont_name"
        ex_eng = ExecutionEngineCommon(self.local, self.xmode)
        status = ex_eng._run_init("2717add4-e6f6-397c-9019-74fa67be439f")
        self.assertEqual(status, "/bin/ls")
        self.assertTrue(mock_chkexec.called)


if __name__ == '__main__':
    main()



#!/usr/bin/env python

# -*- coding: utf-8 -*-
"""
udocker unit tests: GetURLpyCurl
"""

from unittest import TestCase, main
from unittest.mock import patch, Mock
from io import BytesIO as strio
from udocker.utils.curl import GetURLpyCurl
from udocker.config import Config

BUILTINS = "builtins"


class GetURLpyCurlTestCase(TestCase):
    """GetURLpyCurl TestCase."""

    def setUp(self):
        Config().getconf()
        Config().conf['timeout'] = 1
        Config().conf['ctimeout'] = 1
        Config().conf['download_timeout'] = 1
        Config().conf['http_agent'] = ""
        Config().conf['http_proxy'] = ""
        Config().conf['http_insecure'] = 0

    def tearDown(self):
        pass

    def _get(self, *args, **kwargs):
        """Mock for pycurl.get."""
        return args[0]

    def test_01_init(self):
        """Test01 GetURLpyCurl() constructor."""
        geturl = GetURLpyCurl()
        self.assertEqual(geturl._url, None)

    @patch('udocker.utils.curl.pycurl.Curl')
    def test_02_is_available(self, mock_pycurl):
        """Test02 GetURLpyCurl().is_available()."""
        mock_pycurl.return_value = True
        geturl = GetURLpyCurl()
        geturl.is_available()
        self.assertTrue(geturl.is_available())
        self.assertTrue(mock_pycurl.called)

        # mock_pycurl.side_effect = (NameError, AttributeError)
        # geturl = GetURLpyCurl()
        # with self.assertRaises(NameError, AttributeError):
        #     status = geturl.is_available()
        #     self.assertTrue(mock_pycurl.called)
        #     self.assertFalse(geturl.is_available())

        # mock_pycurl.return_value = None
        # geturl = GetURLpyCurl()
        # self.assertFalse(geturl.is_available())

    # def test_03__select_implementation(self):
    #     """Test03 GetURLpyCurl()._select_implementation()."""

    @patch.object(GetURLpyCurl, 'is_available')
    @patch('udocker.utils.curl.Msg')
    @patch('udocker.utils.curl.pycurl.Curl')
    @patch('udocker.utils.curl.CurlHeader')
    def test_04__set_defaults(self, mock_hdr, mock_pyc,
                              mock_msg, mock_selinsec):
        """Test04 GetURLpyCurl()._set_defaults()."""
        mock_selinsec.return_value = True
        mock_msg.level = 0
        mock_msg.VER = 4
        geturl = GetURLpyCurl()
        geturl._set_defaults(mock_pyc, mock_hdr)
        self.assertTrue(mock_pyc.setopt.called)

        # when Msg.level >= Msg.VER = 4: AND insecure
        mock_msg.level = 5
        mock_msg.VER = 4
        geturl = GetURLpyCurl()
        geturl._set_defaults(mock_pyc, mock_hdr)
        self.assertEqual(mock_pyc.setopt.call_count, 18)

        mock_selinsec.return_value = True
        # when Msg.level < Msg.VER = 4: AND secure
        mock_msg.level = 2
        mock_msg.VER = 4
        geturl = GetURLpyCurl()
        geturl._set_defaults(mock_pyc, mock_hdr)
        self.assertEqual(mock_pyc.setopt.call_count, 27)

    @patch('udocker.utils.curl.json.dumps')
    def test_05__mkpycurl(self, mock_jdump):
        """Test05 GetURLpyCurl()._mkpycurl()."""
        curl_patch = patch("udocker.utils.curl.CurlHeader")
        curlhdr = curl_patch.start()
        mock_curlhdr = Mock()
        curlhdr.return_value = mock_curlhdr

        pyc_patch = patch("udocker.utils.curl.pycurl.Curl")
        pycurl = pyc_patch.start()
        mock_pycurl = Mock()
        pycurl.return_value = mock_pycurl

        buff = strio()
        argl = ["http://host"]

        geturl = GetURLpyCurl()
        status = geturl._mkpycurl(pycurl, curlhdr, buff, argl)
        self.assertTrue(pycurl.setopt.called)
        self.assertEqual(status, ("", None))

        kargl = {"post":  "pst1", "sizeonly": True,
                 "proxy": "http://proxy", "ctimeout": 1000,
                 "header": "Authorization: Bearer", "v": True,
                 "nobody": True, "timeout": 50, }
        mock_jdump.return_value = {"post": "pst1"}
        geturl = GetURLpyCurl()
        status = geturl._mkpycurl(pycurl, curlhdr, buff, argl, kargl)
        self.assertTrue(pycurl.setopt.called)
        self.assertTrue(curlhdr.sizeonly)
        self.assertEqual(status, ("", None))

        curlhdr = curl_patch.stop()
        pycurl = pyc_patch.stop()

    @patch.object(GetURLpyCurl, 'is_available')
    def test_06_get(self, mock_sel):
        """Test06 GetURLpyCurl().get() generic get."""
        mock_sel.return_value = True
        geturl = GetURLpyCurl()
        geturl._geturl = type('test', (object,), {})()
        geturl.get = self._get
        status = geturl.get("http://host")
        self.assertEqual(status, "http://host")


if __name__ == '__main__':
    main()



#!/usr/bin/env python
"""
udocker unit tests: Keystore
"""

from unittest import TestCase, main
from unittest.mock import patch, mock_open
from udocker.helper.keystore import KeyStore
from udocker.config import Config

BUILTINS = "builtins"


class KeyStoreTestCase(TestCase):
    """Test KeyStore() local basic credentials storage."""

    def setUp(self):
        Config().getconf()

    def tearDown(self):
        pass

    def test_01_init(self):
        """Test01 KeyStore() constructor."""
        kstore = KeyStore("filename")
        status = kstore.keystore_file
        self.assertEqual(status, "filename")

    # @patch('udocker.helper.keystore.os.path.dirname')
    # @patch('udocker.helper.keystore.HostInfo')
    # @patch('udocker.helper.keystore.FileUtil')
    # def test_02__verify_keystore(self, mock_futil, mock_hinfo, mock_pdir):
    #     """Test02 KeyStore()._verify_keystore()."""
    #     mock_futil.return_value.uid.return_value = 1000
    #     mock_hinfo.return_value.uid.return_value = 1000
    #     mock_pdir.return_value = "somedir/filename"
    #     kstore = KeyStore("filename")
    #     kstore._verify_keystore()
    #     self.assertTrue(mock_futil.uid.called)

    @patch('udocker.helper.keystore.json.load')
    def test_03__read_all(self, mock_jload):
        """Test03 KeyStore()._read_all()."""
        url = u'https://xxx'
        email = u'user@domain'
        auth = u'xxx'
        credentials = {url: {u'email': email, u'auth': auth}}
        mock_jload.return_value = credentials
        with patch(BUILTINS + '.open', mock_open()):
            kstore = KeyStore("filename")
            status = kstore._read_all()
            self.assertEqual(status, credentials)

    @patch.object(KeyStore, '_verify_keystore')
    @patch('udocker.helper.keystore.FileUtil.size')
    def test_04__shred(self, mock_size, mock_verks):
        """Test04 KeyStore()._shred()."""
        mock_verks.return_value = None
        with patch(BUILTINS + '.open', mock_open()):
            kstore = KeyStore("filename")
            status = kstore._shred()
            self.assertEqual(status, 0)

        mock_size.return_value = 123
        mock_verks.return_value = None
        with patch(BUILTINS + '.open', mock_open()):
            kstore = KeyStore("filename")
            status = kstore._shred()
            self.assertEqual(status, 0)

    @patch.object(KeyStore, '_verify_keystore')
    @patch('udocker.helper.keystore.json.dump')
    @patch('udocker.helper.keystore.os.umask')
    def test_05__write_all(self, mock_umask, mock_jdump, mock_verks):
        """Test05 KeyStore()._write_all()."""
        url = u'https://xxx'
        email = u'user@domain'
        auth = u'xxx'
        credentials = {url: {u'email': email, u'auth': auth}}
        mock_umask.return_value = 0o77
        mock_jdump.side_effect = IOError('json dump')
        mock_verks.return_value = None
        with patch(BUILTINS + '.open', mock_open()):
            kstore = KeyStore("filename")
            status = kstore._write_all(credentials)
            self.assertEqual(status, 1)

    @patch.object(KeyStore, '_read_all')
    def test_06_get(self, mock_readall):
        """Test06 KeyStore().get()."""
        url = u'https://xxx'
        email = u'user@domain'
        auth = u'xxx'
        credentials = {url: {u'email': email, u'auth': auth}}
        mock_readall.return_value = credentials
        kstore = KeyStore("filename")
        self.assertTrue(kstore.get(url))
        self.assertFalse(kstore.get("NOT EXISTING ENTRY"))

    @patch.object(KeyStore, '_shred')
    @patch.object(KeyStore, '_write_all')
    @patch.object(KeyStore, '_read_all')
    def test_07_put(self, mock_readall, mock_writeall, mock_shred):
        """Test07 KeyStore().put()."""
        url = u'https://xxx'
        email = u'user@domain'
        auth = u'xxx'
        credentials = {url: {u'email': email, u'auth': auth}}
        mock_shred.return_value = None
        kstore = KeyStore("filename")
        status = kstore.put("", "", "")
        self.assertEqual(status, 1)

        mock_shred.return_value = None
        mock_readall.return_value = dict()
        kstore = KeyStore("filename")
        status = kstore.put(url, auth, email)
        mock_writeall.assert_called_once_with(credentials)

    @patch.object(KeyStore, '_verify_keystore')
    @patch.object(KeyStore, '_shred')
    @patch.object(KeyStore, '_write_all')
    @patch.object(KeyStore, '_read_all')
    def test_08_delete(self, mock_readall, mock_writeall, mock_shred,
                       mock_verks):
        """Test08 KeyStore().delete()."""
        url = u'https://xxx'
        email = u'user@domain'
        auth = u'xxx'
        credentials = {url: {u'email': email, u'auth': auth}}
        mock_readall.return_value = credentials
        mock_writeall.return_value = 0
        mock_shred.return_value = 0
        mock_verks.return_value = None
        kstore = KeyStore("filename")
        status = kstore.delete(url)
        mock_writeall.assert_called_once_with({})
        self.assertEqual(status, 0)

    @patch('udocker.helper.keystore.os.unlink')
    @patch.object(KeyStore, '_verify_keystore')
    @patch.object(KeyStore, '_shred')
    def test_09_erase(self, mock_shred, mock_verks, mock_unlink):
        """Test09 KeyStore().erase()."""
        mock_verks.return_value = None
        mock_shred.return_value = None
        kstore = KeyStore("filename")
        self.assertEqual(kstore.erase(), 0)
        mock_unlink.assert_called_once_with("filename")

        mock_verks.return_value = None
        mock_shred.return_value = None
        mock_unlink.side_effect = IOError
        kstore = KeyStore("filename")
        self.assertEqual(kstore.erase(), 1)


if __name__ == '__main__':
    main()
